#ifndef _UXTHEME_H
#define _UXTHEME_H

#if __POCC__ >= 500
#pragma once
#endif

/* Windows Theme API definitions (Windows XP, Vista) */

#include <commctrl.h>

#if defined(WIN32_DEFAULT_LIBS)
#pragma comment(lib, "uxtheme.lib")
#endif

#ifndef THEMEAPI
#define THEMEAPI         EXTERN_C DECLSPEC_IMPORT HRESULT STDAPICALLTYPE
#define THEMEAPI_(type)  EXTERN_C DECLSPEC_IMPORT type STDAPICALLTYPE
#endif

#if (_WIN32_WINNT >= 0x0600)
#define OTD_FORCE_RECT_SIZING  0x00000001
#define OTD_NONCLIENT  0x00000002
#define OTD_VALIDBITS  (OTD_FORCE_RECT_SIZING|OTD_NONCLIENT)
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#define DTBG_CLIPRECT  0x00000001
#define DTBG_DRAWSOLID  0x00000002
#define DTBG_OMITBORDER  0x00000004
#define DTBG_OMITCONTENT  0x00000008
#define DTBG_COMPUTINGREGION  0x00000010
#define DTBG_MIRRORDC  0x00000020
#define DTBG_NOMIRROR  0x00000040
#define DTBG_VALIDBITS  (DTBG_CLIPRECT|DTBG_DRAWSOLID|DTBG_OMITBORDER|DTBG_OMITCONTENT|DTBG_COMPUTINGREGION|DTBG_MIRRORDC|DTBG_NOMIRROR)

#define DTT_GRAYED  0x1

#define HTTB_BACKGROUNDSEG  0x0000
#define HTTB_FIXEDBORDER  0x0002
#define HTTB_CAPTION  0x0004
#define HTTB_RESIZINGBORDER_LEFT  0x0010
#define HTTB_RESIZINGBORDER_TOP  0x0020
#define HTTB_RESIZINGBORDER_RIGHT  0x0040
#define HTTB_RESIZINGBORDER_BOTTOM  0x0080
#define HTTB_RESIZINGBORDER  (HTTB_RESIZINGBORDER_LEFT|HTTB_RESIZINGBORDER_TOP|HTTB_RESIZINGBORDER_RIGHT|HTTB_RESIZINGBORDER_BOTTOM)
#define HTTB_SIZINGTEMPLATE  0x0100
#define HTTB_SYSTEMSIZINGMARGINS  0x0200

#if WINVER >= 0x0600
#define MAX_INTLIST_COUNT  402
#else
#define MAX_INTLIST_COUNT  10
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#define ETDT_DISABLE  0x00000001
#define ETDT_ENABLE  0x00000002
#define ETDT_USETABTEXTURE  0x00000004
#define ETDT_ENABLETAB  (ETDT_ENABLE|ETDT_USETABTEXTURE)
#define ETDT_ENABLEAEROWIZARDTAB  (ETDT_ENABLE|ETDT_USEAEROWIZARDTABTEXTURE)
#define ETDT_VALIDBITS  (ETDT_DISABLE|ETDT_ENABLE|ETDT_USETABTEXTURE|ETDT_USEAEROWIZARDTABTEXTURE)

#define STAP_ALLOW_NONCLIENT  (1<<0)
#define STAP_ALLOW_CONTROLS  (1<<1)
#define STAP_ALLOW_WEBCONTENT  (1<<2)
#define STAP_VALIDBITS  (STAP_ALLOW_NONCLIENT|STAP_ALLOW_CONTROLS|STAP_ALLOW_WEBCONTENT)

#define SZ_THDOCPROP_DISPLAYNAME  L"DisplayName"
#define SZ_THDOCPROP_CANONICALNAME  L"ThemeName"
#define SZ_THDOCPROP_TOOLTIP  L"ToolTip"
#define SZ_THDOCPROP_AUTHOR  L"author"

#if (_WIN32_WINNT >= 0x0600)
#define WTNCA_NODRAWCAPTION  0x00000001
#define WTNCA_NODRAWICON  0x00000002
#define WTNCA_NOSYSMENU  0x00000004
#define WTNCA_NOMIRRORHELP  0x00000008
#define WTNCA_VALIDBITS  (WTNCA_NODRAWCAPTION|WTNCA_NODRAWICON|WTNCA_NOSYSMENU|WTNCA_NOMIRRORHELP)
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#if (_WIN32_WINNT >= 0x0600)
#define DTPB_WINDOWDC  0x00000001
#define DTPB_USECTLCOLORSTATIC  0x00000002
#define DTPB_USEERASEBKGND  0x00000004
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#if (_WIN32_WINNT >= 0x0600)
#define GBF_DIRECT  0x00000001
#define GBF_COPY  0x00000002
#define GBF_VALIDBITS  (GBF_DIRECT|GBF_COPY)
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#if (_WIN32_WINNT >= 0x0600)
#define BPBF_COMPOSITED  BPBF_TOPDOWNDIB

#define BPPF_ERASE  0x0001
#define BPPF_NOCLIP  0x0002
#define BPPF_NONCLIENT  0x0004

#define BufferedPaintMakeOpaque(hBufferedPaint,prc)  BufferedPaintSetAlpha((hBufferedPaint),(prc),255)
#endif /* #if (_WIN32_WINNT >= 0x0600) */

typedef HANDLE HTHEME;

typedef enum THEMESIZE {
    TS_MIN,
    TS_TRUE,
    TS_DRAW,
};

typedef struct _MARGINS {
    int cxLeftWidth;
    int cxRightWidth;
    int cyTopHeight;
    int cyBottomHeight;
} MARGINS, *PMARGINS;

typedef struct _INTLIST {
    int iValueCount;
    int iValues[MAX_INTLIST_COUNT];
} INTLIST, *PINTLIST;

typedef enum PROPERTYORIGIN {
    PO_STATE,
    PO_PART,
    PO_CLASS,
    PO_GLOBAL,
    PO_NOTFOUND
};

typedef struct _DTBGOPTS {
    DWORD dwSize;
    DWORD dwFlags;
    RECT rcClip;
} DTBGOPTS, *PDTBGOPTS;

#if (_WIN32_WINNT >= 0x0600)
enum WINDOWTHEMEATTRIBUTETYPE {
    WTA_NONCLIENT = 1
};

typedef struct _WTA_OPTIONS {
    DWORD dwFlags;
    DWORD dwMask;
} WTA_OPTIONS, *PWTA_OPTIONS;

typedef HANDLE HPAINTBUFFER;

typedef enum _BP_BUFFERFORMAT {
    BPBF_COMPATIBLEBITMAP,
    BPBF_DIB,
    BPBF_TOPDOWNDIB,
    BPBF_TOPDOWNMONODIB
} BP_BUFFERFORMAT;

typedef enum _BP_ANIMATIONSTYLE {
    BPAS_NONE,
    BPAS_LINEAR,
    BPAS_CUBIC,
    BPAS_SINE
} BP_ANIMATIONSTYLE;

typedef struct _BP_ANIMATIONPARAMS {
    DWORD cbSize;
    DWORD dwFlags;
    BP_ANIMATIONSTYLE style;
    DWORD dwDuration;
} BP_ANIMATIONPARAMS, *PBP_ANIMATIONPARAMS;

typedef struct _BP_PAINTPARAMS {
    DWORD cbSize;
    DWORD dwFlags;
    const RECT *prcExclude;
    const BLENDFUNCTION *pBlendFunction;
} BP_PAINTPARAMS, *PBP_PAINTPARAMS;


typedef HANDLE HANIMATIONBUFFER;
#endif /* #if (_WIN32_WINNT >= 0x0600) */

THEMEAPI_(HTHEME) OpenThemeData(HWND,LPCWSTR);
#if (_WIN32_WINNT >= 0x0600)
THEMEAPI_(HTHEME) OpenThemeDataEx(HWND,LPCWSTR,DWORD);
#endif /* #if (_WIN32_WINNT >= 0x0600) */
THEMEAPI CloseThemeData(HTHEME);
THEMEAPI DrawThemeBackground(HTHEME,HDC,int,int,const RECT*,const RECT*);
THEMEAPI DrawThemeText(HTHEME,HDC,int,int,LPCWSTR,int,DWORD,DWORD,const RECT*);
THEMEAPI GetThemeBackgroundContentRect(HTHEME,HDC,int,int,const RECT*,RECT*);
THEMEAPI GetThemeBackgroundExtent(HTHEME,HDC,int,int,const RECT*,RECT*);
THEMEAPI GetThemePartSize(HTHEME,HDC,int,int,RECT*,enum THEMESIZE,SIZE*);
THEMEAPI GetThemeTextExtent(HTHEME,HDC,int,int,LPCWSTR,int,DWORD,const RECT*,RECT*);
THEMEAPI GetThemeTextMetrics(HTHEME,HDC,int,int,TEXTMETRIC*);
THEMEAPI GetThemeBackgroundRegion(HTHEME,HDC,int,int,const RECT*,HRGN*);
THEMEAPI HitTestThemeBackground(HTHEME,HDC,int,int,DWORD,const RECT*,HRGN,POINT,WORD*);
THEMEAPI DrawThemeEdge(HTHEME,HDC,int,int,const RECT*,UINT,UINT,RECT*);
THEMEAPI DrawThemeIcon(HTHEME,HDC,int,int,const RECT*,HIMAGELIST,int);
THEMEAPI_(BOOL) IsThemePartDefined(HTHEME,int,int);
THEMEAPI_(BOOL) IsThemeBackgroundPartiallyTransparent(HTHEME,int,int);
THEMEAPI GetThemeColor(HTHEME,int,int,int,COLORREF*);
THEMEAPI GetThemeMetric(HTHEME,HDC,int,int,int,int*);
THEMEAPI GetThemeString(HTHEME,int,int,int,LPWSTR,int);
THEMEAPI GetThemeBool(HTHEME,int,int,int,BOOL*);
THEMEAPI GetThemeInt(HTHEME,int,int,int,int*);
THEMEAPI GetThemeEnumValue(HTHEME,int,int,int,int*);
THEMEAPI GetThemePosition(HTHEME,int,int,int,POINT*);
THEMEAPI GetThemeFont(HTHEME,HDC,int,int,int,LOGFONT*);
THEMEAPI GetThemeRect(HTHEME,int,int,int,RECT*);
THEMEAPI GetThemeMargins(HTHEME,HDC,int,int,int,RECT*,MARGINS*);
THEMEAPI GetThemeIntList(HTHEME,int,int,int,INTLIST*);
THEMEAPI GetThemePropertyOrigin(HTHEME,int,int,int,enum PROPERTYORIGIN*);
THEMEAPI SetWindowTheme(HWND,LPCWSTR,LPCWSTR);
THEMEAPI GetThemeFilename(HTHEME,int,int,int,LPWSTR,int);
THEMEAPI_(COLORREF) GetThemeSysColor(HTHEME,int);
THEMEAPI_(HBRUSH) GetThemeSysColorBrush(HTHEME,int);
THEMEAPI_(BOOL) GetThemeSysBool(HTHEME,int);
THEMEAPI_(int) GetThemeSysSize(HTHEME,int);
THEMEAPI GetThemeSysFont(HTHEME,int,LOGFONT*);
THEMEAPI GetThemeSysString(HTHEME,int,LPWSTR,int);
THEMEAPI GetThemeSysInt(HTHEME,int,int*);
THEMEAPI_(BOOL) IsThemeActive(void);
THEMEAPI_(BOOL) IsAppThemed(void);
THEMEAPI_(HTHEME) GetWindowTheme(HWND);
THEMEAPI EnableThemeDialogTexture(HWND,DWORD);
THEMEAPI_(BOOL) IsThemeDialogTextureEnabled(HWND);
THEMEAPI_(DWORD) GetThemeAppProperties(void);
THEMEAPI_(void) SetThemeAppProperties(DWORD);
THEMEAPI GetCurrentThemeName(LPWSTR,int,LPWSTR,int,LPWSTR,int);
THEMEAPI GetThemeDocumentationProperty(LPCWSTR,LPCWSTR,LPWSTR,int);
THEMEAPI DrawThemeParentBackground(HWND,HDC,RECT*);
THEMEAPI EnableTheming(BOOL);
THEMEAPI DrawThemeBackgroundEx(HTHEME,HDC,int,int,const RECT*,const DTBGOPTS*);
#if (_WIN32_WINNT >= 0x0600)
THEMEAPI GetThemeBitmap(HTHEME,int,int,int,ULONG,HBITMAP*);
THEMEAPI GetThemeStream(HTHEME,int,int,int,VOID **,DWORD*,HINSTANCE);
THEMEAPI SetWindowThemeAttribute(HWND,enum WINDOWTHEMEATTRIBUTETYPE,PVOID,DWORD);
inline HRESULT SetWindowThemeNonClientAttributes(HWND hwnd, DWORD dwMask, DWORD dwAttributes) {
    WTA_OPTIONS wta = { .dwMask = dwMask, .dwFlags = dwAttributes };
    return SetWindowThemeAttribute(hwnd, WTA_NONCLIENT, (void*)&wta, sizeof(wta));
}
THEMEAPI DrawThemeParentBackgroundEx(HWND,HDC,DWORD,const RECT*);
THEMEAPI BufferedPaintInit(void);
THEMEAPI BufferedPaintUnInit(void);
THEMEAPI_(HPAINTBUFFER) BeginBufferedPaint(HDC,const RECT*,BP_BUFFERFORMAT,BP_PAINTPARAMS*,HDC*);
THEMEAPI EndBufferedPaint(HPAINTBUFFER,BOOL);
THEMEAPI GetBufferedPaintTargetRect(HPAINTBUFFER,RECT*);
THEMEAPI_(HDC) GetBufferedPaintTargetDC(HPAINTBUFFER);
THEMEAPI_(HDC) GetBufferedPaintDC(HPAINTBUFFER);
THEMEAPI GetBufferedPaintBits(HPAINTBUFFER,RGBQUAD**,int*);
THEMEAPI BufferedPaintClear(HPAINTBUFFER,const RECT*);
THEMEAPI BufferedPaintSetAlpha(HPAINTBUFFER,const RECT*,BYTE);
THEMEAPI BufferedPaintStopAllAnimations(HWND);
THEMEAPI_(HANIMATIONBUFFER) BeginBufferedAnimation(HWND,HDC,const RECT*,BP_BUFFERFORMAT,BP_PAINTPARAMS*,BP_ANIMATIONPARAMS*,HDC*,HDC*);
THEMEAPI EndBufferedAnimation(HANIMATIONBUFFER,BOOL);
THEMEAPI_(BOOL) BufferedPaintRenderAnimation(HWND,HDC);
THEMEAPI_(BOOL) IsCompositionActive(void);
THEMEAPI GetThemeTransitionDuration(HTHEME,int,int,int,int,DWORD*);
#endif /* #if (_WIN32_WINNT >= 0x0600) */

#endif /* _UXTHEME_H */
