/****************************************************************************
 *																			*
 * File    : addin.h														*
 *																			*
 * Purpose : Definitions for Pelles C add-in API 5.0.						*
 *																			*
 * History : Date	   Reason												*
 *			 04-01-28  Created												*
 *			 04-05-19  Added definitions for version 2.80					*
 *			 04-10-12  Added definitions for version 2.90					*
 *			 05-02-25  Added definitions for version 3.00					*
 *			 05-07-30  Added definitions for version 4.00					*
 *			 06-04-29  Added definitions for version 4.50					*
 *			 07-03-05  Added definitions for version 5.00					*
 *			 07-05-10  Added UNICODE interface (for Sven).					*
 *																			*
 ****************************************************************************/

#ifndef _ADDIN_H
#define _ADDIN_H

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define ADDINAPI __declspec(dllexport)

#ifndef SNDMSG
#ifdef __cplusplus
#define SNDMSG ::SendMessage
#else
#define SNDMSG SendMessage
#endif
#endif

/*
 * AddInMain() events.
 */
typedef enum tagADDIN_EVENT {
	AIE_APP_CREATE = 1,		// Main window created (hwnd = main window handle).
	AIE_APP_DESTROY = 2,	// Main window destroyed (hwnd = main window handle).
	AIE_PRJ_CREATE = 3,		// Project pane created (hwnd = project pane handle).
	AIE_PRJ_DESTROY = 4,	// Project pane destroyed (hwnd = project pane handle).
	AIE_PRJ_SAVE = 7,		// Project saved (2.90) (hwnd = project pane handle).
	AIE_PRJ_STARTBUILD = 8,	// Project build will start (2.90) (hwnd = project pane handle).
	AIE_PRJ_ENDBUILD = 9,	// Project build has ended (2.90) (hwnd = project pane handle).
	AIE_DOC_CREATE = 5,		// MDI child window created (hwnd = MDI child window handle).
	AIE_DOC_DESTROY = 6,	// MDI child window destroyed (hwnd = MDI child window handle).
	AIE_DOC_SAVE = 10,		// MDI child window saved (2.90) (hwnd = MDI child window handle).
} ADDIN_EVENT;

/*
 * AddInHelp() events.
 */
typedef enum tagADDIN_HELPEVENT {  /* (4.00) */
	AIHE_SRC_KEYWORD_FIRST = 1,		// First chance: before IDE keyword lookup.
	AIHE_SRC_KEYWORD_SECOND = 2		// Last chance: after IDE keyword lookup, before custom help lookup.
} ADDIN_HELPEVENT;

/*
 * IDE Window types.
 */
enum ADDIN_WINDOW_TYPES {
	AID_UNKNOWN = 0,
	AID_SOURCE = 1,			// Source document.
	AID_RESOURCE = 2,		// Resource script document.
	AID_BITMAP = 3,			// Bitmap document.
	AID_ICON = 4,			// Icon document.
	AID_CURSOR = 5,			// Cursor document.
	AID_ANICURSOR = 6,		// Animated cursor document.
	AID_VIDEO = 7,			// Video document.
	AID_HEXDUMP = 8,		// Hexdump document.
	AID_PROJECT = 9,		// Project pane.
	AID_MSGTABLE = 10,		// Message table document (3.00).
	/*AID_DIALOG,*/
	/*AID_MENU,*/
	/*AID_ACCELTABLE,*/
	/*AID_STRINGTABLE,*/
	/*AID_VERSION,*/
};

/*
 * Text file encodings.
 */
enum ADDIN_TEXT_ENCODING {
	AITE_UNKNOWN = 0,				// Unknown, or don't apply.
	AITE_ANSI = 1,					// ANSI encoding.
	AITE_OEM = 2,					// OEM encoding.
	AITE_UTF8 = 3,					// UTF-8 encoding with BOM.
	AITE_UTF8_NO_BOM = 4,			// UTF-8 encoding without BOM.
	AITE_UTF16_LE = 5,				// UTF-16 little-endian encoding with BOM.
	AITE_UTF16_LE_NO_BOM = 6,		// UTF-16 little-endian encoding without BOM.
	AITE_UTF16_BE = 7,				// UTF-16 big-endian encoding with BOM.
	AITE_UTF16_BE_NO_BOM = 8,		// UTF-16 big-endian encoding without BOM.
	AITE_UTF32_LE = 9,				// UTF-32 little-endian encoding with BOM.
	AITE_UTF32_LE_NO_BOM = 10,		// UTF-32 little-endian encoding without BOM.
	AITE_UTF32_BE = 11,				// UTF-32 big-endian encoding with BOM.
	AITE_UTF32_BE_NO_BOM = 12,		// UTF-32 big-endian encoding without BOM.
};

/*
 * IDE menu identifiers.
 */
enum ADDIN_MENU_IDENTIFIERS {
	AIM_MENU_NOTHING = 0,
	AIM_MENU_FILE = 1,					// File menu.
	AIM_MENU_EDIT = 2,					// Edit menu.
	AIM_MENU_VIEW = 3,					// View menu.
	AIM_MENU_PROJECT = 4,				// Project menu.
	AIM_MENU_PROJECT_CONTEXT = 5,		// Project context menu(s).
	AIM_MENU_HELP = 6,					// Help menu.
	AIM_MENU_SOURCE = 7,				// Source menu.
	AIM_MENU_SOURCE_CONTEXT = AIM_MENU_SOURCE,	// Source context menu.
	AIM_MENU_SOURCE_CONVERT = 9,		// Source convert menu.
	AIM_MENU_RESOURCE = 10,				// Resource menu (3.00).
	AIM_MENU_RESOURCE_NEW = 11,			// New resource menu (3.00).
	AIM_MENU_MSGTABLE = 12,				// Message table menu (3.00).
	AIM_MENU_HELP_WEB = 13,				// Web Help menu (3.00).
};

/*
 * IDE command identifiers - AddIn_SendIDECommand().
 */
enum ADDIN_COMMAND_IDENTIFIERS {
	/* File commands */
	AIC_FILE_NEWPROJECT = 7501,			// File - New project.
	AIC_FILE_NEWSOURCE = 7502,			// File - New source code.
	AIC_FILE_NEWRESOURCE = 7503,		// File - New resource script.
	AIC_FILE_NEWBITMAP = 7504,			// File - New bitmap.
	AIC_FILE_NEWICON = 7505,			// File - New icon.
	AIC_FILE_NEWCURSOR = 7506,			// File - New cursor.
	AIC_FILE_NEWANICURSOR = 7507,		// File - New animated cursor.
	AIC_FILE_NEWVIDEO = 7508,			// File - New video.
	AIC_FILE_NEWHEXDUMP = 7509,			// File - New binary.
	AIC_FILE_NEWMSGTABLE = 7655,		// File - New message table (3.00).
	AIC_FILE_OPEN = 7510,				// File - Open.
	AIC_FILE_CLOSE = 7511,				// File - Close.
	AIC_FILE_CLOSEPROJ = 7721,			// File - Close project (5.00).
	AIC_FILE_SAVE = 7512,				// File - Save.
	AIC_FILE_SAVEAS = 7513,				// File - Save As.
	AIC_FILE_SAVEALL = 7514,			// File - Save All.
	AIC_FILE_PRINT = 7516,				// File - Print.
	AIC_FILE_PRINTSETUP = 7517,			// File - Page setup.
	AIC_FILE_SENDMAIL = 7518,			// File - Send.
	AIC_FILE_PROPERTIES = 7519,			// File - Properties.
	AIC_FILE_EXIT = 7520,				// File - Exit.
	/* Edit commands */
	AIC_EDIT_UNDO = 7521,				// Edit - Undo.
	AIC_EDIT_REDO = 7522,				// Edit - Redo.
	AIC_EDIT_CUT = 7523,				// Edit - Cut.
	AIC_EDIT_COPY = 7524,				// Edit - Copy.
	AIC_EDIT_PASTE = 7525,				// Edit - Paste.
	AIC_EDIT_INSERT = 7526,				// Insert (hidden).
	AIC_EDIT_DELETE = 7527,				// Edit - Delete.
	AIC_EDIT_DELETEWORD = 7650,			// Edit - Delete previous word (2.90).
	AIC_EDIT_SELECTALL = 7528,			// Edit - Select All.
	AIC_EDIT_FIND = 7529,				// Edit - Find.
	AIC_EDIT_FINDNEXT = 7530,			// Edit - Find next.
	AIC_EDIT_REPLACE = 7531,			// Edit - Replace.
	AIC_EDIT_FILEFIND = 7515,			// Edit - Find in files.
	AIC_FILE_FILEFIND = AIC_EDIT_FILEFIND,	// Compatibility name (< 5.00).
	AIC_EDIT_GOTO = 7532,				// Edit - Go To.
	/* View commands */
	AIC_VIEW_LABELS = 7533,				// View - Resource symbols.
	AIC_VIEW_INCLUDES = 7534,			// View - Resource includes.
	AIC_VIEW_SHOWTOOLBAR = 7535,		// View - Toolbar.
	AIC_VIEW_SHOWSTATUSBAR = 7536,		// View - Statusbar.
	AIC_VIEW_SHOWDOCTABS = 7641,		// View - Document tabs (2.80).
	AIC_VIEW_SOURCEFILES = 7722,		// View - Source files (5.00).
	AIC_VIEW_TARGETFILES = 7723,		// View - Target files (5.00).
	AIC_VIEW_PROJECT = 7724,			// View - Project tab (5.00).
	AIC_VIEW_FILEFIND = 7725,			// View - Find in files tab (5.00).
	AIC_VIEW_OUTPUT = 7726,				// View - Output tab (5.00).
	AIC_VIEW_TRACE = 7727,				// View - Debug tab (5.00).
	AIC_VIEW_BREAKPOINTS = 7728,		// View - Breakpoints tab (5.00).
	AIC_VIEW_AUTO = 7729,				// View - Debugger, Auto tab (5.00).
	AIC_VIEW_WATCH = 7730,				// View - Debugger, Watch tab (5.00).
	AIC_VIEW_LOCALS = 7731,				// View - Debugger, Locals tab (5.00).
	AIC_VIEW_GLOBALS = 7732,			// View - Debugger, Globals tab (5.00).
	AIC_VIEW_REGISTERS = 7733,			// View - Debugger, Registers tab (5.00).
	AIC_VIEW_STACK = 7734,				// View - Debugger, Call stack (5.00).
	AIC_VIEW_MEMORY = 7735,				// View - Debugger, Memory tab (5.00).
	AIC_VIEW_THREADS = 7736,			// View - Debugger, Threads tab (5.00).
	AIC_VIEW_MODULES = 7737,			// View - Debugger, Modules tab (5.00).
	AIC_VIEW_EXCEPTIONS = 7738,			// View - Debugger, Exceptions tab (5.00).
	/* Project commands */
	AIC_PROJ_ADDFILE = 7537,			// Project - Add files to project.
	AIC_PROJ_ADDPROJECT = 7667,			// Project - Add project to workspace (4.00).
	AIC_PROJ_SETTINGS = 7538,			// Project - Project options.
	AIC_PROJ_COMPILE = 7539,			// Project - compile.
	AIC_PROJ_BUILD = 7540,				// Project - Build.
	AIC_PROJ_REBUILD = 7541,			// Project - Rebuild.
	AIC_PROJ_BUILDALL = AIC_PROJ_REBUILD,	// Compatibility name (< 5.00).
	AIC_PROJ_BUILDWKS = 7719,			// Project - Build workspace (5.00).
	AIC_PROJ_REBUILDWKS = 7720,			// Project - Rebuild workspace (5.00).
	AIC_PROJ_STOPBUILD = 7542,			// Project - Stop rebuild.
	AIC_PROJ_MAKEALLPREQS = 7543,		// Project - Update all dependencies.
	AIC_PROJ_EXECUTE = 7544,			// Project - Execute.
	AIC_PROJ_ZIP = 7545,				// Project - ZIP project files.
	AIC_PROJ_UNZIP = 7546,				// Project - UNZIP project files.
	AIC_PROJ_SETACTIVE = 7668,			// Project - Set active project (4.00).
	AIC_PROJ_DEPENDENCIES = 7669,		// Project - Dependencies (4.00).
	/* Tool commands */
	AIC_TOOL_CUSTOMIZE = 7547,			// Tools - Customize.
	AIC_TOOL_SETTINGS = 7548,			// Tools - Options.
	AIC_TOOL_CUSTOMTOOL = 7549,			// Tools - Custom tool #1.
	AIC_TOOL_CUSTOMTOOL_1 = 7550,		// Tools - Custom tool #2.
	AIC_TOOL_CUSTOMTOOL_2 = 7551,		// Tools - Custom tool #3.
	AIC_TOOL_CUSTOMTOOL_3 = 7552,		// Tools - Custom tool #4.
	AIC_TOOL_CUSTOMTOOL_4 = 7553,		// Tools - Custom tool #5.
	AIC_TOOL_CUSTOMTOOL_5 = 7554,		// Tools - Custom tool #6.
	AIC_TOOL_CUSTOMTOOL_6 = 7555,		// Tools - Custom tool #7.
	AIC_TOOL_CUSTOMTOOL_7 = 7556,		// Tools - Custom tool #8.
	AIC_TOOL_CUSTOMTOOL_8 = 7711,		// Tools - Custom tool #9 (5.00).
	AIC_TOOL_CUSTOMTOOL_9 = 7712,		// Tools - Custom tool #10 (5.00).
	AIC_TOOL_CUSTOMTOOL_10 = 7713,		// Tools - Custom tool #11 (5.00).
	AIC_TOOL_CUSTOMTOOL_11 = 7714,		// Tools - Custom tool #12 (5.00).
	AIC_TOOL_CUSTOMTOOL_12 = 7715,		// Tools - Custom tool #13 (5.00).
	AIC_TOOL_CUSTOMTOOL_13 = 7716,		// Tools - Custom tool #14 (5.00).
	AIC_TOOL_CUSTOMTOOL_14 = 7717,		// Tools - Custom tool #15 (5.00).
	AIC_TOOL_CUSTOMTOOL_15 = 7718,		// Tools - Custom tool #16 (5.00).
	/* Window commands */
	AIC_WIN_NEW = 7557,					// Window - New window.
	AIC_WIN_CASCADE = 7558,				// Window - Cascade.
	AIC_WIN_TILEVERT = 7559,			// Window - Tile vertically.
	AIC_WIN_TILEHORZ = 7560,			// Window - Tile horizontally.
	AIC_WIN_CLOSEALL = 7561,			// Window - Close All.
	/* Help commands */
	AIC_HELP_CONTENTS = 7562,			// Help - Contents.
	AIC_HELP_ABOUT = 7563,				// Help - About.
	AIC_HELP_CUSTOMHELP = 7564,			// Help - Custom help #1.
	AIC_HELP_CUSTOMHELP_1 = 7565,		// Help - Custom help #2.
	AIC_HELP_CUSTOMHELP_2 = 7566,		// Help - Custom help #3.
	AIC_HELP_CUSTOMHELP_3 = 7567,		// Help - Custom help #4.
	AIC_HELP_CUSTOMHELP_4 = 7568,		// Help - Custom help #5.
	AIC_HELP_CUSTOMHELP_5 = 7569,		// Help - Custom help #6.
	AIC_HELP_CUSTOMHELP_6 = 7570,		// Help - Custom help #7.
	AIC_HELP_CUSTOMHELP_7 = 7571,		// Help - Custom help #8.
	AIC_HELP_CUSTOMHELP_8 = 7703,		// Help - Custom help #9 (5.00).
	AIC_HELP_CUSTOMHELP_9 = 7704,		// Help - Custom help #10 (5.00).
	AIC_HELP_CUSTOMHELP_10 = 7705,		// Help - Custom help #11 (5.00).
	AIC_HELP_CUSTOMHELP_11 = 7706,		// Help - Custom help #12 (5.00).
	AIC_HELP_CUSTOMHELP_12 = 7707,		// Help - Custom help #13 (5.00).
	AIC_HELP_CUSTOMHELP_13 = 7708,		// Help - Custom help #14 (5.00).
	AIC_HELP_CUSTOMHELP_14 = 7709,		// Help - Custom help #15 (5.00).
	AIC_HELP_CUSTOMHELP_15 = 7710,		// Help - Custom help #16 (5.00).
	/* Image commands */
	AIC_IMG_NEWIMAGE = 7572,
	AIC_IMG_DELETEIMAGE = 7573,
	AIC_IMG_LOADPALETTE = 7574,
	AIC_IMG_SAVEPALETTE = 7575,
	AIC_IMG_DEFAULTPALETTE = 7576,
	AIC_IMG_RESIZE = 7761,				// (5.00)
	AIC_IMG_MIRROR_HORZ = 7577,
	AIC_IMG_MIRROR_VERT = 7578,
	AIC_IMG_ROTATE = 7580,
	AIC_IMG_GRAYSCALE = 7579,
	AIC_IMG_SEPIATONE = 7659,			// (4.00)
	AIC_IMG_HSV = 7649,					// (2.90)
	AIC_IMG_OPAQUEALPHA = 7660,			// (4.00)
	AIC_IMG_EFFECT_OILPAINT = 7581,
	AIC_IMG_EFFECT_SPREAD = 7582,
	AIC_IMG_EFFECT_EMBOSS = 7583,
	AIC_IMG_EFFECT_FISHEYE = 7648,		// (2.90)
	AIC_IMG_EFFECT_DROPSHADOW = 7661,	// (4.00)
	AIC_IMG_EFFECT_BLUR = 7584,
	AIC_IMG_EFFECT_SHARPEN = 7585,
	/* Dialog commands */
	AIC_DLG_NEWPICTURE = 7672,			// Dialog - new picture (4.50).
	AIC_DLG_NEWBITMAP = AIC_DLG_NEWPICTURE, // Compatibility name (5.00).
	AIC_DLG_NEWTEXT = 7673,				// Dialog - new text (4.50).
	AIC_DLG_NEWEDITBOX = 7674,			// Dialog - new editbox (4.50).
	AIC_DLG_NEWGROUPBOX = 7675,			// Dialog - new groupbox (4.50).
	AIC_DLG_NEWPUSHBUTTON = 7676,		// Dialog - new pushbutton (4.50).
	AIC_DLG_NEWCHECKBOX = 7677,			// Dialog - new checkbox (4.50).
	AIC_DLG_NEWRADIOBUTTON = 7678,		// Dialog - new radiobutton (4.50).
	AIC_DLG_NEWCOMBOBOX = 7679,			// Dialog - new combobox (4.50).
	AIC_DLG_NEWLISTBOX = 7680,			// Dialog - new listbox (4.50).
	AIC_DLG_NEWHSCROLLBAR = 7681,		// Dialog - new horizontal scrollbar (4.50).
	AIC_DLG_NEWVSCROLLBAR = 7682,		// Dialog - new vertical scrollbar (4.50).
	AIC_DLG_NEWUPDOWN = 7683,			// Dialog - new updown (4.50).
	AIC_DLG_NEWPROGRESS = 7684,			// Dialog - new progress (4.50).
	AIC_DLG_NEWTRACKBAR = 7685,			// Dialog - new trackbar (4.50).
	AIC_DLG_NEWHOTKEY = 7686,			// Dialog - new hotkey (4.50).
	AIC_DLG_NEWLISTVIEW = 7687,			// Dialog - new listview (4.50).
	AIC_DLG_NEWTREEVIEW = 7688,			// Dialog - new treeview (4.50).
	AIC_DLG_NEWTABCONTROL = 7689,		// Dialog - new tabcontrol (4.50).
	AIC_DLG_NEWANIMATE = 7690,			// Dialog - new animate (4.50).
	AIC_DLG_NEWRICHEDIT = 7691,			// Dialog - new richedit 1.0 (4.50).
	AIC_DLG_NEWRICHED20 = 7692,			// Dialog - new richedit 2.0 (4.50).
	AIC_DLG_NEWDATETIME = 7693,			// Dialog - new datetime (4.50).
	AIC_DLG_NEWMONTHCAL = 7694,			// Dialog - new monthcal (4.50).
	AIC_DLG_NEWCOMBOBOXEX = 7695,		// Dialog - new comboboxex (4.50).
	AIC_DLG_NEWREBAR = 7696,			// Dialog - new rebar (4.50).
	AIC_DLG_NEWPAGER = 7697,			// Dialog - new pager (4.50).
	AIC_DLG_NEWTOOLBAR = 7698,			// Dialog - new toolbar (4.50).
	AIC_DLG_NEWSTATUSBAR = 7699,		// Dialog - new statusbar (4.50).
	AIC_DLG_NEWSYSLINK = 7759,			// Dialog - new SysLink (5.00).
	AIC_DLG_NEWIPADDRESS = 7701,		// Dialog - new ip-address (5.00).
	AIC_DLG_NEWNETADDRESS = 7760,		// Dialog - new net-address (5.00).
	AIC_DLG_NEWSPLITBUTTON = 7757,		// Dialog - New splitbutton (5.00).
	AIC_DLG_NEWCOMMANDLINK = 7758,		// Dialog - New commandlink (5.00).
	AIC_DLG_NEWNATIVEFONT = 7702,		// Dialog - new native font (5.00).
	AIC_DLG_ALIGNLEFT = 7586,
	AIC_DLG_ALIGNRIGHT = 7587,
	AIC_DLG_ALIGNTOP = 7588,
	AIC_DLG_ALIGNBOTTOM = 7589,
	AIC_DLG_ALIGNVERT = 7590,
	AIC_DLG_ALIGNHORZ = 7591,
	AIC_DLG_CENTERVERT = 7592,
	AIC_DLG_CENTERHORZ = 7593,
	AIC_DLG_SPACEVERT = 7594,
	AIC_DLG_SPACEHORZ = 7595,
	AIC_DLG_ARRSIZEWIDTH = 7596,
	AIC_DLG_ARRSIZEHEIGHT = 7597,
	AIC_DLG_ARRSIZEBOTH = 7598,
	AIC_DLG_ARRPUSHBOTTOM = 7599,
	AIC_DLG_ARRPUSHRIGHT = 7600,
	AIC_DLG_SIZETOTEXT = 7601,
	AIC_DLG_NEWCUSTOM = 7602,
	AIC_DLG_OPENCUSTOM = 7603,
	AIC_DLG_REMOVECUSTOM = 7604,
	AIC_DLG_LOCKCONTROLS = 7640,		// (2.80)
	AIC_DLG_ORDERCONTROLS = 7652,		// (2.90)
	/* Resource commands */
	AIC_RSRC_NEWBITMAP = 7605,			// Resource - New bitmap.
	AIC_RSRC_NEWICON = 7606,			// Resource - New icon.
	AIC_RSRC_NEWCURSOR = 7607,			// Resource - New cursor.
	AIC_RSRC_NEWANICURSOR = 7608,		// Resource - New animated cursor.
	AIC_RSRC_NEWVIDEO = 7609,			// Resource - New video.
	AIC_RSRC_NEWDIALOG = 7610,			// Resource - New dialog.
	AIC_RSRC_NEWMENU = 7611,			// Resource - New menu.
	AIC_RSRC_NEWSTRINGTABLE = 7612,		// Resource - New string table.
	AIC_RSRC_NEWMSGTABLE = 7654,		// Resource - New message table (3.00).
	AIC_RSRC_NEWACCEL = 7613,			// Resource - New accelerator table.
	AIC_RSRC_NEWVERSION = 7614,			// Resource - New version.
	AIC_RSRC_NEWMANIFEST = 7615,		// Resource - New manifest.
	AIC_RSRC_NEWHTML = 7739,			// Resource - new HTML (5.00).
	AIC_RSRC_NEWCUSTOM = 7653,			// Resource - New custom type (3.00).
	AIC_RSRC_NEWCOPY = 7616,			// Resource - Duplicate.
	AIC_RSRC_IMPORT = 7617,				// Resource - Import.
	AIC_RSRC_SORTNAMES = 7651,			// Resource - Sorted names (2.90).
	/* Message table commands */
	AIC_MESS_SETINCLUDE = 7656,			// Message - Open symbol file (3.00).
	AIC_MESS_VIEWLABELS = 7657,			// Message - View symbols (3.00).
	/* Source commands */
	AIC_SRC_FORCECALLTIP = 7671,		// Source - Show call tip (4.50).
	AIC_SRC_COMPLETEWORD = 7642,		// Source - Complete word (2.80).
	AIC_SRC_INSERTSNIPPET = 7740,		// Source - Insert code snippet (5.00).
	AIC_SRC_SURROUNDBY = 7741,			// Source - surround by #1 (5.00).
	AIC_SRC_SURROUNDBY_1 = 7742,		// Source - surround by #2 (5.00).
	AIC_SRC_SURROUNDBY_2 = 7743,		// Source - surround by #3 (5.00).
	AIC_SRC_SURROUNDBY_3 = 7744,		// Source - surround by #4 (5.00).
	AIC_SRC_SURROUNDBY_4 = 7745,		// Source - surround by #5 (5.00).
	AIC_SRC_SURROUNDBY_5 = 7746,		// Source - surround by #6 (5.00).
	AIC_SRC_SURROUNDBY_6 = 7747,		// Source - surround by #7 (5.00).
	AIC_SRC_SURROUNDBY_7 = 7748,		// Source - surround by #8 (5.00).
	AIC_SRC_SURROUNDBY_8 = 7749,		// Source - surround by #9 (5.00).
	AIC_SRC_SURROUNDBY_9 = 7750,		// Source - surround by #10 (5.00).
	AIC_SRC_SURROUNDBY_10 = 7751,		// Source - surround by #11 (5.00).
	AIC_SRC_SURROUNDBY_11 = 7752,		// Source - surround by #12 (5.00).
	AIC_SRC_SURROUNDBY_12 = 7753,		// Source - surround by #13 (5.00).
	AIC_SRC_SURROUNDBY_13 = 7754,		// Source - surround by #14 (5.00).
	AIC_SRC_SURROUNDBY_14 = 7755,		// Source - surround by #15 (5.00).
	AIC_SRC_SURROUNDBY_15 = 7756,		// Source - surround by #16 (5.00).
	AIC_SRC_UCASE = 7618,				// Source - Convert to upper case.
	AIC_SRC_LCASE = 7619,				// Source - Convert to lower case.
	AIC_SRC_TABTOSPACE = 7620,			// Source - Convert to spaces (from tabs).
	AIC_SRC_SPACETOTAB = 7621,			// Source - Convert to tabs (from spaces).
	AIC_SRC_FORMAT = 7658,				// Source - Convert to formatted C code (3.00).
	AIC_SRC_COMMENT = 7662,				// Source - Convert to comment (4.00).
	AIC_SRC_UNCOMMENT = 7663,			// Source - Convert to no comment (4.00).
	AIC_SRC_NEXTBOOKMARK = 7622,		// Source - Next bookmark.
	AIC_SRC_PREVBOOKMARK = 7645,		// Source - Previous bookmark.
	AIC_SRC_SETBOOKMARK = 7623,			// Source - Toggle bookmark.
	AIC_SRC_CLEARBOOKMARKS = 7624,		// Source - Remove all bookmarks.
	AIC_SRC_WHITESPACE = 7625,			// Source - View whitespace.
	AIC_SRC_LINEBREAKS = 7626,			// Source - View breaks.
	AIC_SRC_LINENUMBERS = 7627,			// Source - View linenumbers.
	AIC_SRC_FOLDINGS = 7644,			// Source - View foldings (2.90).
	AIC_SRC_MATCHINGBRACES = 7700,		// Source - View matching braces (4.50).
	AIC_SRC_FOLDALL = 7646,				// Source - Fold all (2.90).
	AIC_SRC_UNFOLDALL = 7647,			// Source - Unfold all (2.90).
	/* Debug commands */
	AIC_DBG_GO = 7628,
	AIC_DBG_RESTART = 7629,
	AIC_DBG_STOP = 7630,
	AIC_DBG_BREAK = 7631,
	AIC_DBG_STEPINTO = 7632,
	AIC_DBG_STEPOVER = 7633,
	AIC_DBG_STEPOUT = 7665,				// (4.00)
	AIC_DBG_RUNTOCURSOR = 7666,			// (4.00)
	AIC_DBG_TOGGLEBREAKPOINT = 7634,
	AIC_DBG_SETBREAKPOINT = AIC_DBG_TOGGLEBREAKPOINT,  // Compatibility
	AIC_DBG_SHOWNEXTSTMT = 7635,
	AIC_DBG_ADDWATCH = 7643,			// (2.80)
	AIC_DBG_QUICKWATCH = 7664,			// (4.00)
	AIC_DBG_HWBREAKS = 7670,			// (4.00)
	/* Misc commands */
	AIC_MISC_OPEN = 7636,
	AIC_MISC_TEST = 7637,
	AIC_MISC_CHECKMNEMONICS = 7638,
	AIC_MISC_NEXTWINDOW = 7639,
};

/*
 * IDE message identifiers.
 */
enum ADDIN_MESSAGE_IDENTIFIERS {
	/* Application window messages */
	AIM_ADD_COMMAND_A = (WM_APP+1000),
	AIM_ADD_COMMAND_W = (WM_APP+1062),  /* (5.00) */
	AIM_REMOVE_COMMAND = (WM_APP+1001),
	AIM_GET_ACTIVE_DOCUMENT = (WM_APP+1002),
	AIM_NEW_DOCUMENT = (WM_APP+1003),
	AIM_OPEN_DOCUMENT_A = (WM_APP+1004),
	AIM_OPEN_DOCUMENT_W = (WM_APP+1063),  /* (5.00) */
	AIM_ENUM_DOCUMENTS = (WM_APP+1046),  /* (2.90) */
	AIM_ADD_TAB_PAGE_A = (WM_APP+1005),
	AIM_ADD_TAB_PAGE_W = (WM_APP+1064),  /* (5.00) */
	AIM_REMOVE_TAB_PAGE = (WM_APP+1006),
	AIM_SELECT_TAB_PAGE = (WM_APP+1007),
	AIM_WRITE_OUTPUT_A = (WM_APP+1041),  /* (2.90) */
	AIM_WRITE_OUTPUT_W = (WM_APP+1065),  /* (5.00) */
	AIM_ADD_FILE_TYPE_A = (WM_APP+1042),  /* (2.90) */
	AIM_ADD_FILE_TYPE_W = (WM_APP+1066),  /* (5.00) */
	AIM_FIND_IN_FILES_W = (WM_APP+1092),  /* (5.00) */
	AIM_GET_FOLDER_W = (WM_APP+1093),  /* (5.00) */
	/* Project window messages */
	AIM_GET_PROJECT_TYPE = (WM_APP+1094),  /* (5.00) */
	AIM_ADD_PROJECT_FILE_A = (WM_APP+1008),
	AIM_ADD_PROJECT_FILE_W = (WM_APP+1067),  /* (5.00) */
	AIM_DELETE_PROJECT_FILE_A = (WM_APP+1009),
	AIM_DELETE_PROJECT_FILE_W = (WM_APP+1068),  /* (5.00) */
	AIM_RENAME_PROJECT_FILE_A = (WM_APP+1060),  /* (4.00) */
	AIM_RENAME_PROJECT_FILE_W = (WM_APP+1069),  /* (5.00) */
	AIM_EXCLUDE_PROJECT_FILE_A = (WM_APP+1061),  /* (4.00) */
	AIM_EXCLUDE_PROJECT_FILE_W = (WM_APP+1070),  /* (5.00) */
	AIM_ENUM_PROJECT_FILES_A = (WM_APP+1043),  /* (2.90) */
	AIM_ENUM_PROJECT_FILES_W = (WM_APP+1071),  /* (5.00) */
	AIM_GET_PROJECT_SYMBOL_A = (WM_APP+1010),
	AIM_GET_PROJECT_SYMBOL_W = (WM_APP+1072),  /* (5.00) */
	AIM_SET_PROJECT_SYMBOL_A = (WM_APP+1011),
	AIM_SET_PROJECT_SYMBOL_W = (WM_APP+1073),  /* (5.00) */
	AIM_GET_PROJECT_SHELLS_A = (WM_APP+1012),
	AIM_GET_PROJECT_SHELLS_W = (WM_APP+1074),  /* (5.00) */
	AIM_SET_PROJECT_SHELLS_A = (WM_APP+1013),
	AIM_SET_PROJECT_SHELLS_W = (WM_APP+1075),  /* (5.00) */
	AIM_DID_PROJECT_BUILD_FAIL = (WM_APP+1045),  /* (2.90) */
	AIM_GET_ACTIVE_PROJECT_A = (WM_APP+1052),  /* (4.00) */
	AIM_GET_ACTIVE_PROJECT_W = (WM_APP+1076),  /* (5.00) */
	AIM_SET_ACTIVE_PROJECT_A = (WM_APP+1053),  /* (4.00) */
	AIM_SET_ACTIVE_PROJECT_W = (WM_APP+1077),  /* (5.00) */
	AIM_ADD_WORKSPACE_PROJECT_A = (WM_APP+1054),  /* (4.00) */
	AIM_ADD_WORKSPACE_PROJECT_W = (WM_APP+1078),  /* (5.00) */
	AIM_DELETE_WORKSPACE_PROJECT_A = (WM_APP+1055),  /* (4.00) */
	AIM_DELETE_WORKSPACE_PROJECT_W = (WM_APP+1079),  /* (5.00) */
	AIM_ENUM_WORKSPACE_PROJECTS_A = (WM_APP+1056),  /* (4.00) */
	AIM_ENUM_WORKSPACE_PROJECTS_W = (WM_APP+1080),  /* (5.00) */
	AIM_SET_PROJECT_BREAKPOINT_A = (WM_APP+1057),  /* (4.00) */
	AIM_SET_PROJECT_BREAKPOINT_W = (WM_APP+1081),  /* (5.00) */
	AIM_REMOVE_PROJECT_BREAKPOINT_A = (WM_APP+1058),  /* (4.00) */
	AIM_REMOVE_PROJECT_BREAKPOINT_W = (WM_APP+1082),  /* (5.00) */
	AIM_ENUM_PROJECT_BREAKPOINTS_A = (WM_APP+1059),  /* (4.00) */
	AIM_ENUM_PROJECT_BREAKPOINTS_W = (WM_APP+1083),  /* (5.00) */
	/* Document window messages */
	AIM_GET_DOCUMENT_TYPE = (WM_APP+1014),
	AIM_GET_DOCUMENT_INFO_A = (WM_APP+1015),
	AIM_GET_DOCUMENT_INFO_W = (WM_APP+1084),  /* (5.00) */
	AIM_SET_DOCUMENT_INFO_A = (WM_APP+1016),
	AIM_SET_DOCUMENT_INFO_W = (WM_APP+1085),  /* (5.00) */
	AIM_CAN_DOCUMENT_UNDO = (WM_APP+1017),
	AIM_DOCUMENT_UNDO = (WM_APP+1018),
	AIM_FORGET_DOCUMENT_UNDO = (WM_APP+1019),
	AIM_SET_BREAKPOINT = (WM_APP+1020),
	AIM_REMOVE_BREAKPOINTS = (WM_APP+1021),
	AIM_ENUM_BREAKPOINTS = (WM_APP+1044),  /* (2.90) [OBSOLETE] */
	/* Source document window messages */
	AIM_GET_SOURCE_TEXT_A = (WM_APP+1022),
	AIM_GET_SOURCE_TEXT_W = (WM_APP+1086),  /* (5.00) */
	AIM_GET_SOURCE_TEXT_LENGTH = (WM_APP+1023),
	AIM_SET_SOURCE_TEXT_A = (WM_APP+1024),
	AIM_SET_SOURCE_TEXT_W = (WM_APP+1087),  /* (5.00) */
	AIM_GET_SOURCE_LINE_A = (WM_APP+1025),
	AIM_GET_SOURCE_LINE_W = (WM_APP+1088),  /* (5.00) */
	AIM_GET_SOURCE_LINE_LENGTH = (WM_APP+1026),
	AIM_GET_SOURCE_SELECTION_TEXT_A = (WM_APP+1027),
	AIM_GET_SOURCE_SELECTION_TEXT_W = (WM_APP+1089),  /* (5.00) */
	AIM_REPLACE_SOURCE_SELECTION_TEXT_A = (WM_APP+1028),
	AIM_REPLACE_SOURCE_SELECTION_TEXT_W = (WM_APP+1090),  /* (5.00) */
	AIM_GET_SOURCE_SELECTION = (WM_APP+1029),
	AIM_SET_SOURCE_SELECTION = (WM_APP+1030),
	AIM_GET_SOURCE_LINE_COUNT = (WM_APP+1031),
	AIM_SOURCE_LINE_INDEX = (WM_APP+1032),
	AIM_FIRST_VISIBLE_SOURCE_LINE = (WM_APP+1033),
	AIM_SOURCE_LINE_FROM_CHAR = (WM_APP+1034),
	AIM_SOURCE_CHAR_FROM_POS = (WM_APP+1035),
	AIM_SOURCE_POS_FROM_CHAR = (WM_APP+1036),
	AIM_SCROLL_SOURCE_CARET = (WM_APP+1037),
	AIM_FIND_SOURCE_TEXT_A = (WM_APP+1038),
	AIM_FIND_SOURCE_TEXT_W = (WM_APP+1091),  /* (5.00) */
	AIM_GET_SOURCE_INFO = (WM_APP+1039),
	AIM_SET_SOURCE_INFO = (WM_APP+1040),
	/* Resource document window messages */
	AIM_ENUM_RESOURCES = (WM_APP+1047),  /* (3.00) */
	AIM_FIND_RESOURCE = (WM_APP+1048),  /* (3.00) */
	AIM_ADD_RESOURCE = (WM_APP+1049),  /* (3.00) */
	AIM_DELETE_RESOURCE = (WM_APP+1050),  /* (3.00) */
	AIM_UPDATE_RESOURCE = (WM_APP+1051),  /* (3.00) */
#if defined(UNICODE) || defined(_UNICODE)
	AIM_ADD_COMMAND = AIM_ADD_COMMAND_W,
	AIM_OPEN_DOCUMENT = AIM_OPEN_DOCUMENT_W,
	AIM_ADD_TAB_PAGE = AIM_ADD_TAB_PAGE_W,
	AIM_WRITE_OUTPUT = AIM_WRITE_OUTPUT_W,
	AIM_ADD_FILE_TYPE = AIM_ADD_FILE_TYPE_W,
	AIM_FIND_IN_FILES = AIM_FIND_IN_FILES_W,
	AIM_GET_FOLDER = AIM_GET_FOLDER_W,
	AIM_ADD_PROJECT_FILE = AIM_ADD_PROJECT_FILE_W,
	AIM_DELETE_PROJECT_FILE = AIM_DELETE_PROJECT_FILE_W,
	AIM_RENAME_PROJECT_FILE = AIM_RENAME_PROJECT_FILE_W,
	AIM_EXCLUDE_PROJECT_FILE = AIM_EXCLUDE_PROJECT_FILE_W,
	AIM_ENUM_PROJECT_FILES = AIM_ENUM_PROJECT_FILES_W,
	AIM_GET_PROJECT_SYMBOL = AIM_GET_PROJECT_SYMBOL_W,
	AIM_SET_PROJECT_SYMBOL = AIM_SET_PROJECT_SYMBOL_W,
	AIM_GET_PROJECT_SHELLS = AIM_GET_PROJECT_SHELLS_W,
	AIM_SET_PROJECT_SHELLS = AIM_SET_PROJECT_SHELLS_W,
	AIM_GET_ACTIVE_PROJECT = AIM_GET_ACTIVE_PROJECT_W,
	AIM_SET_ACTIVE_PROJECT = AIM_SET_ACTIVE_PROJECT_W,
	AIM_ADD_WORKSPACE_PROJECT = AIM_ADD_WORKSPACE_PROJECT_W,
	AIM_DELETE_WORKSPACE_PROJECT = AIM_DELETE_WORKSPACE_PROJECT_W,
	AIM_ENUM_WORKSPACE_PROJECTS = AIM_ENUM_WORKSPACE_PROJECTS_W,
	AIM_SET_PROJECT_BREAKPOINT = AIM_SET_PROJECT_BREAKPOINT_W,
	AIM_REMOVE_PROJECT_BREAKPOINT = AIM_REMOVE_PROJECT_BREAKPOINT_W,
	AIM_ENUM_PROJECT_BREAKPOINTS = AIM_ENUM_PROJECT_BREAKPOINTS_W,
	AIM_GET_DOCUMENT_INFO = AIM_GET_DOCUMENT_INFO_W,
	AIM_SET_DOCUMENT_INFO = AIM_SET_DOCUMENT_INFO_W,
	AIM_GET_SOURCE_TEXT = AIM_GET_SOURCE_TEXT_W,
	AIM_SET_SOURCE_TEXT = AIM_SET_SOURCE_TEXT_W,
	AIM_GET_SOURCE_LINE = AIM_GET_SOURCE_LINE_W,
	AIM_GET_SOURCE_SELECTION_TEXT = AIM_GET_SOURCE_SELECTION_TEXT_W,
	AIM_REPLACE_SOURCE_SELECTION_TEXT = AIM_REPLACE_SOURCE_SELECTION_TEXT_W,
	AIM_FIND_SOURCE_TEXT = AIM_FIND_SOURCE_TEXT_W,
#else /* !(UNICODE || _UNICODE) */
	AIM_ADD_COMMAND = AIM_ADD_COMMAND_A,
	AIM_OPEN_DOCUMENT = AIM_OPEN_DOCUMENT_A,
	AIM_ADD_TAB_PAGE = AIM_ADD_TAB_PAGE_A,
	AIM_WRITE_OUTPUT = AIM_WRITE_OUTPUT_A,
	AIM_ADD_FILE_TYPE = AIM_ADD_FILE_TYPE_A,
	/* AIM_FIND_IN_FILES = AIM_FIND_IN_FILES_A, */
	/* AIM_GET_FOLDER = AIM_GET_FOLDER_A, */
	AIM_ADD_PROJECT_FILE = AIM_ADD_PROJECT_FILE_A,
	AIM_DELETE_PROJECT_FILE = AIM_DELETE_PROJECT_FILE_A,
	AIM_RENAME_PROJECT_FILE = AIM_RENAME_PROJECT_FILE_A,
	AIM_EXCLUDE_PROJECT_FILE = AIM_EXCLUDE_PROJECT_FILE_A,
	AIM_ENUM_PROJECT_FILES = AIM_ENUM_PROJECT_FILES_A,
	AIM_GET_PROJECT_SYMBOL = AIM_GET_PROJECT_SYMBOL_A,
	AIM_SET_PROJECT_SYMBOL = AIM_SET_PROJECT_SYMBOL_A,
	AIM_GET_PROJECT_SHELLS = AIM_GET_PROJECT_SHELLS_A,
	AIM_SET_PROJECT_SHELLS = AIM_SET_PROJECT_SHELLS_A,
	AIM_GET_ACTIVE_PROJECT = AIM_GET_ACTIVE_PROJECT_A,
	AIM_SET_ACTIVE_PROJECT = AIM_SET_ACTIVE_PROJECT_A,
	AIM_ADD_WORKSPACE_PROJECT = AIM_ADD_WORKSPACE_PROJECT_A,
	AIM_DELETE_WORKSPACE_PROJECT = AIM_DELETE_WORKSPACE_PROJECT_A,
	AIM_ENUM_WORKSPACE_PROJECTS = AIM_ENUM_WORKSPACE_PROJECTS_A,
	AIM_SET_PROJECT_BREAKPOINT = AIM_SET_PROJECT_BREAKPOINT_A,
	AIM_REMOVE_PROJECT_BREAKPOINT = AIM_REMOVE_PROJECT_BREAKPOINT_A,
	AIM_ENUM_PROJECT_BREAKPOINTS = AIM_ENUM_PROJECT_BREAKPOINTS_A,
	AIM_GET_DOCUMENT_INFO = AIM_GET_DOCUMENT_INFO_A,
	AIM_SET_DOCUMENT_INFO = AIM_SET_DOCUMENT_INFO_A,
	AIM_GET_SOURCE_TEXT = AIM_GET_SOURCE_TEXT_A,
	AIM_SET_SOURCE_TEXT = AIM_SET_SOURCE_TEXT_A,
	AIM_GET_SOURCE_LINE = AIM_GET_SOURCE_LINE_A,
	AIM_GET_SOURCE_SELECTION_TEXT = AIM_GET_SOURCE_SELECTION_TEXT_A,
	AIM_REPLACE_SOURCE_SELECTION_TEXT = AIM_REPLACE_SOURCE_SELECTION_TEXT_A,
	AIM_FIND_SOURCE_TEXT = AIM_FIND_SOURCE_TEXT_A,
#endif /* !(UNICODE || _UNICODE) */
};

typedef struct tagADDIN_ADD_COMMAND_A {
	UINT cbSize;						// Size of this structure (version control).
	int id; 							// Command identifier.
	LPCSTR pszText;						// Display name.
	HICON hIcon;						// Display icon.
	int idMenu;							// Menu id, or AIM_MENU_NOTHING for button (enum ADDIN_MENU_IDENTIFIERS) (2.90).
} ADDIN_ADD_COMMAND_A, *PADDIN_ADD_COMMAND_A;

typedef struct tagADDIN_ADD_COMMAND_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	int id; 							// Command identifier.
	LPCWSTR pszText;					// Display name.
	HICON hIcon;						// Display icon.
	int idMenu;							// Menu id, or AIM_MENU_NOTHING for button (enum ADDIN_MENU_IDENTIFIERS).
} ADDIN_ADD_COMMAND_W, *PADDIN_ADD_COMMAND_W;

typedef struct tagADDIN_DOCUMENT_INFO_A {
	UINT cbSize;						// Size of this structure (version control).
	int nType;							// Document type (enum ADDIN_WINDOW_TYPES).
	BOOL fDirty;						// Is the document dirty?
	FILETIME ftLastWriteTime;			// When was the document last written?
	CHAR szFilename[260];				// Document file name (empty string if no name).
} ADDIN_DOCUMENT_INFO_A, *PADDIN_DOCUMENT_INFO_A;

typedef struct tagADDIN_DOCUMENT_INFO_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	int nType;							// Document type (enum ADDIN_WINDOW_TYPES).
	int nEncoding;						// Document encoding (enum ADDIN_TEXT_ENCODING).
	BOOL fDirty;						// Is the document dirty?
	FILETIME ftLastWriteTime;			// When was the document last written?
	WCHAR szFilename[260];				// Document file name (empty string if no name).
} ADDIN_DOCUMENT_INFO_W, *PADDIN_DOCUMENT_INFO_W;

typedef struct tagADDIN_SOURCE_INFO {
	UINT cbSize;						// Size of the structure (version control).
	BOOL fReadOnly; 					// Is the document read-only?
	BOOL fInsertMode;					// Is the source editor in insert mode?
	BOOL fCRBreak;						// <CR> is part of line breaks?
	BOOL fLFBreak;						// <LF> is part of line breaks?
	BOOL fWhiteSpace;					// Show white-space (global setting)?
	BOOL fBreaks;						// Show line breaks (global setting)?
	BOOL fLineNumbers;					// Show line numbers (global setting)?
	BOOL fFoldings;						// Show foldings (global setting)? (2.90)
	BOOL fCallTips;						// Use call tips (global setting)? (2.90)
	BOOL fMemberList;					// Use member list (global setting)? (2.90)
	BOOL fDragDrop;						// Use drag-and-drop editing (global setting)? (2.90)
	BOOL fCtrlChars;					// Accept control characters when typing (global setting)? (2.90)
	BOOL fWrapFindQuery;				// Question before search wrap (global setting)? (2.90)
	BOOL fSpaceTabs;					// Insert spaces for tabs (global setting)? (3.00)
	int nIndentMode;					// Indentation type: 0-none, 1-block, 2-smart (global setting). (3.00)
	int cSmartLines;					// Number of smart indent context lines (global setting). (3.00)
	int nCStyle;						// C formatting style: 0 or 1 (global setting). (3.00)
	BOOL fMatchingBraces;				// Show matching braces (global setting)? (4.50)
} ADDIN_SOURCE_INFO, *PADDIN_SOURCE_INFO;

typedef struct tagADDIN_RANGE {
	int iStartPos;						// Starting position.
	int iEndPos;						// Ending position.
} ADDIN_RANGE, *PADDIN_RANGE;

typedef struct tagADDIN_ENUM_DOCUMENTS {
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(HWND hwndDoc, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_DOCUMENTS, *PADDIN_ENUM_DOCUMENTS;

typedef struct tagADDIN_ENUM_PROJECT_FILES_A {
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(LPCSTR pszFilename, LPVOID pvData);  // Callback function.
	enum _ADDIN_ENUM_PROJECT_FILES_FLAGS {
		EPFF_TARGET_FILES = 0x0001,
		EPFF_DEPENDENT_FILES = 0x0002,
		EPFF_ALL_FILES = EPFF_TARGET_FILES|EPFF_DEPENDENT_FILES
	} fuFlags;							// Flags.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_PROJECT_FILES_A, *PADDIN_ENUM_PROJECT_FILES_A;

typedef struct tagADDIN_ENUM_PROJECT_FILES_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(LPCWSTR pszFilename, LPVOID pvData);  // Callback function.
	enum _ADDIN_ENUM_PROJECT_FILES_FLAGS fuFlags;	// Flags.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_PROJECT_FILES_W, *PADDIN_ENUM_PROJECT_FILES_W;

typedef struct tagADDIN_ENUM_BREAKPOINTS {  /* [OBSOLETE] */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(int iLine, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_BREAKPOINTS, *PADDIN_ENUM_BREAKPOINTS;

typedef struct tagADDIN_ENUM_WORKSPACE_PROJECTS_A {  /* (4.00) */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(LPCSTR pszName, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_WORKSPACE_PROJECTS_A, *PADDIN_ENUM_WORKSPACE_PROJECTS_A;

typedef struct tagADDIN_ENUM_WORKSPACE_PROJECTS_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(LPCWSTR pszName, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_WORKSPACE_PROJECTS_W, *PADDIN_ENUM_WORKSPACE_PROJECTS_W;

typedef struct tagADDIN_PROJECT_BREAKPOINT_A {  /* (4.00) */
	UINT cbSize;						// Size of this structure (version control).
	LPCSTR pszFilename;					// Source file name.
	LPCSTR pszExpression;				// Break when this expression is true (if != NULL).
	int cHitCount;						// Break when this number of hits (if > 1).
	int nLine;							// Line number (from 1).
	BOOL fEnabled;						// Enabled?
} ADDIN_PROJECT_BREAKPOINT_A, *PADDIN_PROJECT_BREAKPOINT_A;

typedef struct tagADDIN_PROJECT_BREAKPOINT_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	LPCWSTR pszFilename;				// Source file name.
	LPCWSTR pszExpression;				// Break when this expression is true (if != NULL).
	int cHitCount;						// Break when this number of hits (if > 1).
	int nLine;							// Line number (from 1).
	BOOL fEnabled;						// Enabled?
} ADDIN_PROJECT_BREAKPOINT_W, *PADDIN_PROJECT_BREAKPOINT_W;

typedef struct tagADDIN_ENUM_PROJECT_BREAKPOINTS_A {  /* (4.00) */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(const ADDIN_PROJECT_BREAKPOINT_A * const pBrkpnt, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_PROJECT_BREAKPOINTS_A, *PADDIN_ENUM_PROJECT_BREAKPOINTS_A;

typedef struct tagADDIN_ENUM_PROJECT_BREAKPOINTS_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(const ADDIN_PROJECT_BREAKPOINT_W * const pBrkpnt, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_PROJECT_BREAKPOINTS_W, *PADDIN_ENUM_PROJECT_BREAKPOINTS_W;

typedef enum tagADDIN_PARSER_COLORS {
	ADDIN_COLOR_TEXT = 0,				// Normal text.
	ADDIN_COLOR_KEYWORD = 2,			// Keyword.
	ADDIN_COLOR_COMMENT = 3,			// Comment.
	ADDIN_COLOR_NUMBER = 4,				// Number.
	ADDIN_COLOR_STRING = 5,				// String literal.
	ADDIN_COLOR_PREPROCESSOR = 6,		// Preprocessor directive.
	ADDIN_COLOR_FUNCTION = 7,			// Function.
	ADDIN_COLOR_OPERATOR = 8,			// Operator.
} ADDIN_PARSER_COLORS;

typedef struct tagADDIN_PARSE_POINT {
	int iChar;							// Column position (from zero) where text changes...
	int iColor;							// ...to this color (enum ADDIN_PARSER_COLORS).
} ADDIN_PARSE_POINT;

typedef struct tagADDIN_ADD_FILE_TYPE_A {  /* [OBSOLETE] */
	UINT cbSize;						// Size of this structure (version control).
	LPCSTR pszDescription;				// File description.
	LPCSTR pszExtension;				// File extension (without a dot, like "BAS").
	USHORT (CALLBACK *pfnParser)(USHORT usCookie, LPCSTR pszText, int cchText, ADDIN_PARSE_POINT aPoints[], PINT pcPoints);  // Source code parser callback, or NULL.
	DWORD reserved;						// (Reserved).
	LPCSTR pszShells;					// Build command(s) - "command 1\0command 2\0...\0" (3.00)
} ADDIN_ADD_FILE_TYPE_A, *PADDIN_ADD_FILE_TYPE_A;

typedef struct tagADDIN_ADD_FILE_TYPE_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	LPCWSTR pszDescription;				// File description.
	LPCWSTR pszExtension;				// File extension (without a dot, like "BAS").
	USHORT (CALLBACK *pfnParser)(USHORT usCookie, LPCWSTR pszText, int cchText, ADDIN_PARSE_POINT aPoints[], PINT pcPoints);  // Source code parser callback, or NULL.
	DWORD reserved;						// (Reserved).
	LPCWSTR pszShells;					// Build command(s) - "command 1\0command 2\0...\0" (3.00)
} ADDIN_ADD_FILE_TYPE_W, *PADDIN_ADD_FILE_TYPE_W;

// Parser helper macros.
#define ADDIN_MAKE_COOKIE(flags,level)  MAKEWORD((flags),(level))
#define ADDIN_GET_COOKIE_LEVEL(cookie)  HIBYTE(cookie)
#define ADDIN_GET_COOKIE_FLAGS(cookie)  LOBYTE(cookie)

// Resource type ordinals.
enum tagADDIN_RSRC_ORDINALS {
	ADDIN_RSRC_CURSOR = 101,	// Bundled cursor; like a cursor file, not a cursor resource.
	ADDIN_RSRC_BITMAP = 2,
	ADDIN_RSRC_ICON = 100,		// Bundled icon; like an icon file, not an icon resource.
	ADDIN_RSRC_MENU = 4,
	ADDIN_RSRC_DIALOG = 5,
	ADDIN_RSRC_STRING = 6,
	ADDIN_RSRC_ACCELERATOR = 9,
	ADDIN_RSRC_RCDATA = 10,
	ADDIN_RSRC_MESSAGETABLE = 11,
	ADDIN_RSRC_VERSION = 16,
	ADDIN_RSRC_PLUGPLAY = 19,	// (obsolete says M$)
	ADDIN_RSRC_VXD = 20,		// (obsolete says M$)
	ADDIN_RSRC_ANICURSOR = 21,
	ADDIN_RSRC_ANIICON = 22,
	ADDIN_RSRC_HTML = 23,
	ADDIN_RSRC_MANIFEST = 24,
};

typedef struct tagADDIN_RESOURCE {
	UINT cbSize;						// Size of this structure (version control).
	LPCWSTR pwchName;					// Name identifier (string or ordinal).
	LPCWSTR pwchType;					// Type identifier (string or ordinal).
	LPCVOID pvData;						// Raw data.
	DWORD cbData;						// Size of raw data.
	WORD wLanguageId;					// Language identifier.
	WORD wReserved;
} ADDIN_RESOURCE, *PADDIN_RESOURCE;

typedef struct tagADDIN_ENUM_RESOURCES {
	UINT cbSize;						// Size of this structure (version control).
	BOOL (CALLBACK *pfnCallback)(const ADDIN_RESOURCE * const pResource, LPVOID pvData);  // Callback function.
	LPVOID pvData;						// User data for callback.
} ADDIN_ENUM_RESOURCES, *PADDIN_ENUM_RESOURCES;

typedef struct tagADDIN_FIND_IN_FILES_W {  /* (5.00) */
	UINT cbSize;						// Size of this structure (version control).
	LPCWSTR pszText;					// Text to search for.
	LPCWSTR pszPath;					// Full path to folder to search in.
	LPCWSTR pszFormat;					// List of extensions to search through ("*.ex1;*.ex2;...").
	BOOL fMatchCase;					// Match case?
	BOOL fRecurse;						// Recurse into subfolders?
} ADDIN_FIND_IN_FILES_W, *PADDIN_FIND_IN_FILES_W;

enum tagADDIN_FOLDER {
	ADDIN_FOLDER_BIN = 1,				// Like: '<path-to-pelles-c>\Bin'.
	ADDIN_FOLDER_ADDINS = 2,			// Like: '<path-to-pelles-c>\Bin\AddIns'.
	ADDIN_FOLDER_HELP = 3,				// Like: '<path-to-pelles-c>\Bin\Help'.
	ADDIN_FOLDER_INTL = 4,				// Like: '<path-to-pelles-c>\Bin\Intl'.
	ADDIN_FOLDER_WIZARDS = 5,			// Like: '<path-to-pelles-c>\Bin\Wizards'.
	ADDIN_FOLDER_LIB = 6,				// Like: '<path-to-pelles-c>\Lib'.
	ADDIN_FOLDER_INCLUDE = 7,			// Like: '<path-to-pelles-c>\Include'.
};

// Project types.
typedef enum tagADDIN_PROJECT_TYPE {
	ADDIN_PROJTYPE_WIN32_GUI = 0,
	ADDIN_PROJTYPE_WIN32_DLL = 1,
	ADDIN_PROJTYPE_WIN32_LIBRARY = 2,
	ADDIN_PROJTYPE_WIN32_CONSOLE = 3,
#ifdef _WIN64
	ADDIN_PROJTYPE_WIN64_GUI = 10,
	ADDIN_PROJTYPE_WIN64_DLL = 11,
	ADDIN_PROJTYPE_WIN64_LIBRARY = 12,
	ADDIN_PROJTYPE_WIN64_CONSOLE = 13,
#endif /* _WIN64 */
	ADDIN_PROJTYPE_WINCE_POCKET_PC_GUI = 4,
	ADDIN_PROJTYPE_WINCE_POCKET_PC_DLL = 5,
	ADDIN_PROJTYPE_WINCE_POCKET_PC_LIBRARY = 6,
	ADDIN_PROJTYPE_WINCE_SMARTPHONE_GUI = 7,
	ADDIN_PROJTYPE_WINCE_SMARTPHONE_DLL = 8,
	ADDIN_PROJTYPE_WINCE_SMARTPHONE_LIBRARY = 9,
	ADDIN_PROJTYPE_WIN32_INSTALLER = 50,
	ADDIN_PROJTYPE_WINCE_POCKET_PC_INSTALLER = 51,
	ADDIN_PROJTYPE_WINCE_SMARTPHONE_INSTALLER = 52,
} ADDIN_PROJECT_TYPE, *PADDIN_PROJECT_TYPE;

/****** Message API ********************************************************/

#define AddIn_SendIDECommand(hwnd,id)  (void)SNDMSG((hwnd),WM_COMMAND,MAKEWPARAM((UINT)(id),0),0)
#define AddIn_AddCommandA(hwnd,pAddCmd)  (BOOL)SNDMSG((hwnd),AIM_ADD_COMMAND_A,0,(LPARAM)(PADDIN_ADD_COMMAND_A)(pAddCmd))
#define AddIn_AddCommandW(hwnd,pAddCmd)  (BOOL)SNDMSG((hwnd),AIM_ADD_COMMAND_W,0,(LPARAM)(PADDIN_ADD_COMMAND_W)(pAddCmd))
#define AddIn_RemoveCommand(hwnd,id)  (BOOL)SNDMSG((hwnd),AIM_REMOVE_COMMAND,(WPARAM)(id),0)
#define AddIn_GetActiveDocument(hwnd)  (HWND)SNDMSG((hwnd),AIM_GET_ACTIVE_DOCUMENT,0,0)
#define AddIn_NewDocument(hwnd,nType)  (HWND)SNDMSG((hwnd),AIM_NEW_DOCUMENT,(WPARAM)(nType),0)
#define AddIn_OpenDocumentA(hwnd,nType,pszFile)  (HWND)SNDMSG((hwnd),AIM_OPEN_DOCUMENT_A,(WPARAM)(nType),(LPARAM)(LPCSTR)(pszFile))
#define AddIn_OpenDocumentW(hwnd,nType,pszFile)  (HWND)SNDMSG((hwnd),AIM_OPEN_DOCUMENT_W,(WPARAM)(nType),(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_CloseDocument(hwnd)  (void)SNDMSG((hwnd),WM_CLOSE,0,0)
#define AddIn_EnumDocuments(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_DOCUMENTS,0,(LPARAM)(PADDIN_ENUM_DOCUMENTS)(pEnum))
#define AddIn_AddTabPageA(hwnd,pszTitle,hwndPage)  (BOOL)SNDMSG((hwnd),AIM_ADD_TAB_PAGE_A,(WPARAM)(LPCSTR)(pszTitle),(LPARAM)(HWND)(hwndPage))
#define AddIn_AddTabPageW(hwnd,pszTitle,hwndPage)  (BOOL)SNDMSG((hwnd),AIM_ADD_TAB_PAGE_W,(WPARAM)(LPCWSTR)(pszTitle),(LPARAM)(HWND)(hwndPage))
#define AddIn_RemoveTabPage(hwnd,hwndPage)  (BOOL)SNDMSG((hwnd),AIM_REMOVE_TAB_PAGE,0,(LPARAM)(HWND)(hwndPage))
#define AddIn_SelectTabPage(hwnd,hwndPage)  (BOOL)SNDMSG((hwnd),AIM_SELECT_TAB_PAGE,0,(LPARAM)(hwndPage))
#define AddIn_ClearOutput(hwnd)  (BOOL)SNDMSG((hwnd),AIM_WRITE_OUTPUT,0,0)
#define AddIn_WriteOutputA(hwnd,pszText)  (BOOL)SNDMSG((hwnd),AIM_WRITE_OUTPUT_A,0,(LPARAM)(LPCSTR)(pszText))
#define AddIn_WriteOutputW(hwnd,pszText)  (BOOL)SNDMSG((hwnd),AIM_WRITE_OUTPUT_W,0,(LPARAM)(LPCWSTR)(pszText))
#define AddIn_AddFileTypeA(hwnd,pAddFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_FILE_TYPE_A,0,(LPARAM)(PADDIN_ADD_FILE_TYPE_A)(pAddFile))
#define AddIn_AddFileTypeW(hwnd,pAddFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_FILE_TYPE_W,0,(LPARAM)(PADDIN_ADD_FILE_TYPE_W)(pAddFile))
#define AddIn_FindInFilesW(hwnd,pFind)  (BOOL)SNDMSG((hwnd),AIM_FIND_IN_FILES_W,0,(LPARAM)(PADDIN_FIND_IN_FILES_W)(pFind))
#define AddIn_GetIDEFolderW(hwnd,eFolder,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_FOLDER_W,(WPARAM)(eFolder),(LPARAM)(LPWSTR)(pch))))
/**/
#define AddIn_AddCommandA(hwnd,pAddCmd)  (BOOL)SNDMSG((hwnd),AIM_ADD_COMMAND_A,0,(LPARAM)(PADDIN_ADD_COMMAND_A)(pAddCmd))
#define AddIn_AddCommandW(hwnd,pAddCmd)  (BOOL)SNDMSG((hwnd),AIM_ADD_COMMAND_W,0,(LPARAM)(PADDIN_ADD_COMMAND_W)(pAddCmd))
#define AddIn_RemoveCommand(hwnd,id)  (BOOL)SNDMSG((hwnd),AIM_REMOVE_COMMAND,(WPARAM)(id),0)
#define AddIn_GetProjectType(hwnd,pType)  (int)SNDMSG((hwnd),AIM_GET_PROJECT_TYPE,0,(LPARAM)(PADDIN_PROJECT_TYPE)(pType))
#define AddIn_AddProjectFileA(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_PROJECT_FILE_A,0,(LPARAM)(LPCSTR)(pszFile))
#define AddIn_AddProjectFileW(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_PROJECT_FILE_W,0,(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_DeleteProjectFileA(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_DELETE_PROJECT_FILE_A,0,(LPARAM)(LPCSTR)(pszFile))
#define AddIn_DeleteProjectFileW(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_DELETE_PROJECT_FILE_W,0,(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_RenameProjectFileA(hwnd,pszOldFile,pszNewFile)  (BOOL)SNDMSG((hwnd),AIM_RENAME_PROJECT_FILE_A,(WPARAM)(LPCSTR)(pszOldFile),(LPARAM)(LPCSTR)(pszNewFile))
#define AddIn_RenameProjectFileW(hwnd,pszOldFile,pszNewFile)  (BOOL)SNDMSG((hwnd),AIM_RENAME_PROJECT_FILE_W,(WPARAM)(LPCWSTR)(pszOldFile),(LPARAM)(LPCWSTR)(pszNewFile))
#define AddIn_ExcludeProjectFileA(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_EXCLUDE_PROJECT_FILE_A,(WPARAM)(1),(LPARAM)(LPCSTR)(pszFile))
#define AddIn_ExcludeProjectFileW(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_EXCLUDE_PROJECT_FILE_W,(WPARAM)(1),(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_IncludeProjectFileA(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_EXCLUDE_PROJECT_FILE_A,(WPARAM)(0),(LPARAM)(LPCSTR)(pszFile))
#define AddIn_IncludeProjectFileW(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_EXCLUDE_PROJECT_FILE_W,(WPARAM)(0),(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_GetProjectSymbolA(hwnd,pszName,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_PROJECT_SYMBOL_A,(WPARAM)(LPCSTR)(pszName),(LPARAM)(LPSTR)(pch))))
#define AddIn_GetProjectSymbolW(hwnd,pszName,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_PROJECT_SYMBOL_W,(WPARAM)(LPCWSTR)(pszName),(LPARAM)(LPWSTR)(pch))))
#define AddIn_SetProjectSymbolA(hwnd,pszName,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_SYMBOL_A,(WPARAM)(LPCSTR)(pszName),(LPARAM)(LPCSTR)(psz))
#define AddIn_SetProjectSymbolW(hwnd,pszName,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_SYMBOL_W,(WPARAM)(LPCWSTR)(pszName),(LPARAM)(LPCWSTR)(psz))
#define AddIn_GetProjectShellsA(hwnd,pszFile,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_PROJECT_SHELLS_A,(WPARAM)(LPCSTR)(pszFile),(LPARAM)(LPSTR)(pch))))
#define AddIn_GetProjectShellsW(hwnd,pszFile,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_PROJECT_SHELLS_W,(WPARAM)(LPCWSTR)(pszFile),(LPARAM)(LPWSTR)(pch))))
#define AddIn_SetProjectShellsA(hwnd,pszFile,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_SHELLS_A,(WPARAM)(LPCSTR)(pszFile),(LPARAM)(LPCSTR)(psz))
#define AddIn_SetProjectShellsW(hwnd,pszFile,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_SHELLS_W,(WPARAM)(LPCWSTR)(pszFile),(LPARAM)(LPCWSTR)(psz))
#define AddIn_EnumProjectFilesA(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_PROJECT_FILES_A,0,(LPARAM)(PADDIN_ENUM_PROJECT_FILES_A)(pEnum))
#define AddIn_EnumProjectFilesW(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_PROJECT_FILES_W,0,(LPARAM)(PADDIN_ENUM_PROJECT_FILES_W)(pEnum))
#define AddIn_DidProjectBuildFail(hwnd)  (BOOL)SNDMSG((hwnd),AIM_DID_PROJECT_BUILD_FAIL,0,0)
#define AddIn_GetActiveProjectA(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_ACTIVE_PROJECT_A,(WPARAM)(int)(cchMax),(LPARAM)(LPSTR)(pch))
#define AddIn_GetActiveProjectW(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_ACTIVE_PROJECT_W,(WPARAM)(int)(cchMax),(LPARAM)(LPWSTR)(pch))
#define AddIn_SetActiveProjectA(hwnd,pszName)  (BOOL)SNDMSG((hwnd),AIM_SET_ACTIVE_PROJECT_A,0,(LPARAM)(LPCSTR)(pszName))
#define AddIn_SetActiveProjectW(hwnd,pszName)  (BOOL)SNDMSG((hwnd),AIM_SET_ACTIVE_PROJECT_W,0,(LPARAM)(LPCWSTR)(pszName))
#define AddIn_AddWorkspaceProjectA(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_WORKSPACE_PROJECT_A,0,(LPARAM)(LPCSTR)(pszFile))
#define AddIn_AddWorkspaceProjectW(hwnd,pszFile)  (BOOL)SNDMSG((hwnd),AIM_ADD_WORKSPACE_PROJECT_W,0,(LPARAM)(LPCWSTR)(pszFile))
#define AddIn_DeleteWorkspaceProjectA(hwnd,pszName)  (BOOL)SNDMSG((hwnd),AIM_DELETE_WORKSPACE_PROJECT_A,0,(LPARAM)(LPCSTR)(pszName))
#define AddIn_DeleteWorkspaceProjectW(hwnd,pszName)  (BOOL)SNDMSG((hwnd),AIM_DELETE_WORKSPACE_PROJECT_W,0,(LPARAM)(LPCWSTR)(pszName))
#define AddIn_EnumWorkspaceProjectsA(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_WORKSPACE_PROJECTS_A,0,(LPARAM)(PADDIN_ENUM_WORKSPACE_PROJECTS_A)(pEnum))
#define AddIn_EnumWorkspaceProjectsW(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_WORKSPACE_PROJECTS_W,0,(LPARAM)(PADDIN_ENUM_WORKSPACE_PROJECTS_W)(pEnum))
#define AddIn_SetProjectBreakpointA(hwnd,pBrkpnt)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_BREAKPOINT_A,0,(LPARAM)(PADDIN_PROJECT_BREAKPOINT_A)(pBrkpnt))
#define AddIn_SetProjectBreakpointW(hwnd,pBrkpnt)  (BOOL)SNDMSG((hwnd),AIM_SET_PROJECT_BREAKPOINT_W,0,(LPARAM)(PADDIN_PROJECT_BREAKPOINT_W)(pBrkpnt))
#define AddIn_RemoveProjectBreakpointA(hwnd,pBrkpnt)  (BOOL)SNDMSG((hwnd),AIM_REMOVE_PROJECT_BREAKPOINT_A,0,(LPARAM)(PADDIN_PROJECT_BREAKPOINT_A)(pBrkpnt))
#define AddIn_RemoveProjectBreakpointW(hwnd,pBrkpnt)  (BOOL)SNDMSG((hwnd),AIM_REMOVE_PROJECT_BREAKPOINT_W,0,(LPARAM)(PADDIN_PROJECT_BREAKPOINT_W)(pBrkpnt))
#define AddIn_EnumProjectBreakpointsA(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_PROJECT_BREAKPOINTS_A,0,(LPARAM)(PADDIN_ENUM_PROJECT_BREAKPOINTS_A)(pEnum))
#define AddIn_EnumProjectBreakpointsW(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_PROJECT_BREAKPOINTS_W,0,(LPARAM)(PADDIN_ENUM_PROJECT_BREAKPOINTS_W)(pEnum))
/**/
#define AddIn_GetDocumentType(hwnd)  (int)SNDMSG((hwnd),AIM_GET_DOCUMENT_TYPE,0,0)
#define AddIn_GetDocumentInfoA(hwnd,pDocInfo)  (BOOL)SNDMSG((hwnd),AIM_GET_DOCUMENT_INFO_A,0,(LPARAM)(PADDIN_DOCUMENT_INFO_A)(pDocInfo))
#define AddIn_GetDocumentInfoW(hwnd,pDocInfo)  (BOOL)SNDMSG((hwnd),AIM_GET_DOCUMENT_INFO_W,0,(LPARAM)(PADDIN_DOCUMENT_INFO_W)(pDocInfo))
#define AddIn_SetDocumentInfoA(hwnd,pDocInfo)  (BOOL)SNDMSG((hwnd),AIM_SET_DOCUMENT_INFO_A,0,(LPARAM)(PADDIN_DOCUMENT_INFO_A)(pDocInfo))
#define AddIn_SetDocumentInfoW(hwnd,pDocInfo)  (BOOL)SNDMSG((hwnd),AIM_SET_DOCUMENT_INFO_W,0,(LPARAM)(PADDIN_DOCUMENT_INFO_W)(pDocInfo))
#define AddIn_CanDocumentUndo(hwnd)  (BOOL)SNDMSG((hwnd),AIM_CAN_DOCUMENT_UNDO,0,0)
#define AddIn_DocumentUndo(hwnd)  (void)SNDMSG((hwnd),AIM_DOCUMENT_UNDO,0,0)
#define AddIn_ForgetDocumentUndo(hwnd)  (void)SNDMSG((hwnd),AIM_FORGET_DOCUMENT_UNDO,0,0)
#define AddIn_SetBreakpoint(hwnd,line)  (BOOL)SNDMSG((hwnd),AIM_SET_BREAKPOINT,(WPARAM)(int)(line),0)
#define AddIn_RemoveBreakpoints(hwnd)  (void)SNDMSG((hwnd),AIM_REMOVE_BREAKPOINTS,0,0)
#define AddIn_EnumBreakpoints(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_BREAKPOINTS,0,(LPARAM)(PADDIN_ENUM_BREAKPOINTS)(pEnum))
/**/
#define AddIn_GetSourceTextA(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_TEXT_A,(WPARAM)(int)(cchMax),(LPARAM)(LPSTR)(pch))
#define AddIn_GetSourceTextW(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_TEXT_W,(WPARAM)(int)(cchMax),(LPARAM)(LPWSTR)(pch))
#define AddIn_GetSourceTextLength(hwnd)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_TEXT_LENGTH,0,0)
#define AddIn_SetSourceTextA(hwnd,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_SOURCE_TEXT_A,0,(LPARAM)(LPCSTR)(psz))
#define AddIn_SetSourceTextW(hwnd,psz)  (BOOL)SNDMSG((hwnd),AIM_SET_SOURCE_TEXT_W,0,(LPARAM)(LPCWSTR)(psz))
#define AddIn_GetSourceLineA(hwnd,line,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_SOURCE_LINE_A,(WPARAM)(int)(line),(LPARAM)(LPSTR)(pch))))
#define AddIn_GetSourceLineW(hwnd,line,pch,cchMax)  ((*((int *)(pch))=(cchMax)), ((int)SNDMSG((hwnd),AIM_GET_SOURCE_LINE_W,(WPARAM)(int)(line),(LPARAM)(LPWSTR)(pch))))
#define AddIn_GetSourceLineLength(hwnd,line)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_LINE_LENGTH,(WPARAM)(int)(line),0)
#define AddIn_GetSourceSelTextA(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_SELECTION_TEXT_A,(WPARAM)(int)(cchMax),(LPARAM)(LPSTR)(pch))
#define AddIn_GetSourceSelTextW(hwnd,pch,cchMax)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_SELECTION_TEXT_W,(WPARAM)(int)(cchMax),(LPARAM)(LPWSTR)(pch))
#define AddIn_ReplaceSourceSelTextA(hwnd,psz)  (void)SNDMSG((hwnd),AIM_REPLACE_SOURCE_SELECTION_TEXT_A,0,(LPARAM)(LPCSTR)(psz))
#define AddIn_ReplaceSourceSelTextW(hwnd,psz)  (void)SNDMSG((hwnd),AIM_REPLACE_SOURCE_SELECTION_TEXT_W,0,(LPARAM)(LPCWSTR)(psz))
#define AddIn_GetSourceSel(hwnd,pRange)  (BOOL)SNDMSG((hwnd),AIM_GET_SOURCE_SELECTION,0,(LPARAM)(PADDIN_RANGE)(pRange))
#define AddIn_SetSourceSel(hwnd,pRange)  (BOOL)SNDMSG((hwnd),AIM_SET_SOURCE_SELECTION,0,(LPARAM)(PADDIN_RANGE)(pRange))
#define AddIn_GetSourceLineCount(hwnd)  (int)SNDMSG((hwnd),AIM_GET_SOURCE_LINE_COUNT,0,0)
#define AddIn_SourceLineIndex(hwnd,line)  (int)SNDMSG((hwnd),AIM_SOURCE_LINE_INDEX,(WPARAM)(int)(line),0)
#define AddIn_FirstVisibleSourceLine(hwnd)  (int)SNDMSG((hwnd),AIM_FIRST_VISIBLE_SOURCE_LINE,0,0)
#define AddIn_SourceLineFromChar(hwnd,ich)  (int)SNDMSG((hwnd),AIM_SOURCE_LINE_FROM_CHAR,(WPARAM)(int)(ich),0)
#define AddIn_SourceCharFromPos(hwnd,ppt)  (int)SNDMSG((hwnd),AIM_SOURCE_CHAR_FROM_POS,0,(LPARAM)(POINT*)(ppt))
#define AddIn_SourcePosFromChar(hwnd,ich,ppt)  (void)SNDMSG((hwnd),AIM_SOURCE_POS_FROM_CHAR,(WPARAM)(int)(ich),(LPARAM)(POINT*)(ppt))
#define AddIn_ScrollSourceCaret(hwnd)  (void)SNDMSG((hwnd),AIM_SCROLL_SOURCE_CARET,0,0)
#define AddIn_FindSourceTextA(hwnd,fuFlags,pszText)  (int)SNDMSG((hwnd),AIM_FIND_SOURCE_TEXT_A,(WPARAM)(UINT)(fuFlags),(LPARAM)(LPCSTR)(pszText))
#define AddIn_FindSourceTextW(hwnd,fuFlags,pszText)  (int)SNDMSG((hwnd),AIM_FIND_SOURCE_TEXT_W,(WPARAM)(UINT)(fuFlags),(LPARAM)(LPCSTR)(pszText))
#define AddIn_GetSourceInfo(hwnd,pSrcInfo)  (BOOL)SNDMSG((hwnd),AIM_GET_SOURCE_INFO,0,(LPARAM)(PADDIN_SOURCE_INFO)(pSrcInfo))
#define AddIn_SetSourceInfo(hwnd,pSrcInfo)  (BOOL)SNDMSG((hwnd),AIM_SET_SOURCE_INFO,0,(LPARAM)(PADDIN_SOURCE_INFO)(pSrcInfo))
/**/
#define AddIn_EnumResources(hwnd,pEnum)  (BOOL)SNDMSG((hwnd),AIM_ENUM_RESOURCES,0,(LPARAM)(PADDIN_ENUM_RESOURCES)(pEnum))
#define AddIn_FindResource(hwnd,pResource)  (BOOL)SNDMSG((hwnd),AIM_FIND_RESOURCE,0,(LPARAM)(PADDIN_RESOURCE)(pResource))
#define AddIn_AddResource(hwnd,pResource)  (BOOL)SNDMSG((hwnd),AIM_ADD_RESOURCE,0,(LPARAM)(PADDIN_RESOURCE)(pResource))
#define AddIn_DeleteResource(hwnd,pResource)  (BOOL)SNDMSG((hwnd),AIM_DELETE_RESOURCE,0,(LPARAM)(PADDIN_RESOURCE)(pResource))
#define AddIn_UpdateResource(hwnd,pResource)  (BOOL)SNDMSG((hwnd),AIM_UPDATE_RESOURCE,0,(LPARAM)(PADDIN_RESOURCE)(pResource))

#if defined(UNICODE) || defined(_UNICODE)
typedef ADDIN_ADD_COMMAND_W ADDIN_ADD_COMMAND;
typedef ADDIN_DOCUMENT_INFO_W ADDIN_DOCUMENT_INFO;
typedef ADDIN_ENUM_PROJECT_FILES_W ADDIN_ENUM_PROJECT_FILES;
typedef ADDIN_ENUM_WORKSPACE_PROJECTS_W ADDIN_ENUM_WORKSPACE_PROJECTS;
typedef ADDIN_PROJECT_BREAKPOINT_W ADDIN_PROJECT_BREAKPOINT;
typedef ADDIN_ENUM_PROJECT_BREAKPOINTS_W ADDIN_ENUM_PROJECT_BREAKPOINTS;
typedef ADDIN_ADD_FILE_TYPE_W ADDIN_ADD_FILE_TYPE;
typedef ADDIN_FIND_IN_FILES_W ADDIN_FIND_IN_FILES;
#define AddIn_AddCommand  AddIn_AddCommandW
#define AddIn_OpenDocument  AddIn_OpenDocumentW
#define AddIn_AddTabPage  AddIn_AddTabPageW
#define AddIn_WriteOutput  AddIn_WriteOutputW
#define AddIn_AddFileType  AddIn_AddFileTypeW
#define AddIn_AddProjectFile  AddIn_AddProjectFileW
#define AddIn_DeleteProjectFile  AddIn_DeleteProjectFileW
#define AddIn_RenameProjectFile  AddIn_RenameProjectFileW
#define AddIn_ExcludeProjectFile  AddIn_ExcludeProjectFileW
#define AddIn_IncludeProjectFile  AddIn_IncludeProjectFileW
#define AddIn_GetProjectSymbol  AddIn_GetProjectSymbolW
#define AddIn_SetProjectSymbol  AddIn_SetProjectSymbolW
#define AddIn_GetProjectShells  AddIn_GetProjectShellsW
#define AddIn_SetProjectShells  AddIn_SetProjectShellsW
#define AddIn_EnumProjectFiles  AddIn_EnumProjectFilesW
#define AddIn_GetActiveProject  AddIn_GetActiveProjectW
#define AddIn_SetActiveProject  AddIn_SetActiveProjectW
#define AddIn_AddWorkspaceProject  AddIn_AddWorkspaceProjectW
#define AddIn_DeleteWorkspaceProject  AddIn_DeleteWorkspaceProjectW
#define AddIn_EnumWorkspaceProjects  AddIn_EnumWorkspaceProjectsW
#define AddIn_SetProjectBreakpoint  AddIn_SetProjectBreakpointW
#define AddIn_RemoveProjectBreakpoint  AddIn_RemoveProjectBreakpointW
#define AddIn_EnumProjectBreakpoints  AddIn_EnumProjectBreakpointsW
#define AddIn_GetDocumentInfo  AddIn_GetDocumentInfoW
#define AddIn_SetDocumentInfo  AddIn_SetDocumentInfoW
#define AddIn_GetSourceText  AddIn_GetSourceTextW
#define AddIn_SetSourceText  AddIn_SetSourceTextW
#define AddIn_GetSourceLine  AddIn_GetSourceLineW
#define AddIn_GetSourceSelText  AddIn_GetSourceSelTextW
#define AddIn_ReplaceSourceSelText  AddIn_ReplaceSourceSelTextW
#define AddIn_FindSourceText  AddIn_FindSourceTextW
#define AddIn_FindInFiles  AddIn_FindInFilesW
#define AddIn_GetIDEFolder  AddIn_GetIDEFolderW
#else /* !(UNICODE || _UNICODE) */
typedef ADDIN_ADD_COMMAND_A ADDIN_ADD_COMMAND;
typedef ADDIN_DOCUMENT_INFO_A ADDIN_DOCUMENT_INFO;
typedef ADDIN_ENUM_PROJECT_FILES_A ADDIN_ENUM_PROJECT_FILES;
typedef ADDIN_ENUM_WORKSPACE_PROJECTS_A ADDIN_ENUM_WORKSPACE_PROJECTS;
typedef ADDIN_PROJECT_BREAKPOINT_A ADDIN_PROJECT_BREAKPOINT;
typedef ADDIN_ENUM_PROJECT_BREAKPOINTS_A ADDIN_ENUM_PROJECT_BREAKPOINTS;
typedef ADDIN_ADD_FILE_TYPE_A ADDIN_ADD_FILE_TYPE;
#define AddIn_AddCommand  AddIn_AddCommandA
#define AddIn_OpenDocument  AddIn_OpenDocumentA
#define AddIn_AddTabPage  AddIn_AddTabPageA
#define AddIn_WriteOutput  AddIn_WriteOutputA
#define AddIn_AddFileType  AddIn_AddFileTypeA
#define AddIn_AddProjectFile  AddIn_AddProjectFileA
#define AddIn_DeleteProjectFile  AddIn_DeleteProjectFileA
#define AddIn_RenameProjectFile  AddIn_RenameProjectFileA
#define AddIn_ExcludeProjectFile  AddIn_ExcludeProjectFileA
#define AddIn_IncludeProjectFile  AddIn_IncludeProjectFileA
#define AddIn_GetProjectSymbol  AddIn_GetProjectSymbolA
#define AddIn_SetProjectSymbol  AddIn_SetProjectSymbolA
#define AddIn_GetProjectShells  AddIn_GetProjectShellsA
#define AddIn_SetProjectShells  AddIn_SetProjectShellsA
#define AddIn_EnumProjectFiles  AddIn_EnumProjectFilesA
#define AddIn_GetActiveProject  AddIn_GetActiveProjectA
#define AddIn_SetActiveProject  AddIn_SetActiveProjectA
#define AddIn_AddWorkspaceProject  AddIn_AddWorkspaceProjectA
#define AddIn_DeleteWorkspaceProject  AddIn_DeleteWorkspaceProjectA
#define AddIn_EnumWorkspaceProjects  AddIn_EnumWorkspaceProjectsA
#define AddIn_SetProjectBreakpoint  AddIn_SetProjectBreakpointA
#define AddIn_RemoveProjectBreakpoint  AddIn_RemoveProjectBreakpointA
#define AddIn_EnumProjectBreakpoints  AddIn_EnumProjectBreakpointsA
#define AddIn_GetDocumentInfo  AddIn_GetDocumentInfoA
#define AddIn_SetDocumentInfo  AddIn_SetDocumentInfoA
#define AddIn_GetSourceText  AddIn_GetSourceTextA
#define AddIn_SetSourceText  AddIn_SetSourceTextA
#define AddIn_GetSourceLine  AddIn_GetSourceLineA
#define AddIn_GetSourceSelText  AddIn_GetSourceSelTextA
#define AddIn_ReplaceSourceSelText  AddIn_ReplaceSourceSelTextA
#define AddIn_FindSourceText  AddIn_FindSourceTextA
#endif /* !(UNICODE || _UNICODE) */

/****** Function prototypes ************************************************/

ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent);  /* required export */
ADDINAPI void WINAPI AddInCommand(int idCmd);  /* optional [OBSOLETE] */
ADDINAPI void WINAPI AddInSetup(HWND hwnd, LPCVOID pcvData);  /* optional (2.90) */
#if defined(UNICODE) || defined(_UNICODE)
ADDINAPI void WINAPI AddInCommandExW(int idCmd, LPCVOID pcvData);  /* optional (2.90) */
ADDINAPI BOOL WINAPI AddInHelpW(HWND hwnd, ADDIN_HELPEVENT eEvent, LPCVOID pcvData);  /* optional (4.00) */
#define AddInCommandEx  AddInCommandExW
#define AddInHelp  AddInHelpW
#else /* !(UNICODE || _UNICODE) */
ADDINAPI void WINAPI AddInCommandEx(int idCmd, LPCVOID pcvData);  /* optional (2.90) */
ADDINAPI BOOL WINAPI AddInHelp(HWND hwnd, ADDIN_HELPEVENT eEvent, LPCVOID pcvData);  /* optional (4.00) */
#endif /* !(UNICODE || _UNICODE) */

/****** Inline helper functions ********************************************/

static inline BOOL IsResourceOrdinalW(PCWSTR pwch) { return (pwch[0] == (WCHAR)-1); }
static inline int GetResourceOrdinalW(PCWSTR pwch) { return (pwch[0] == (WCHAR)-1) ? pwch[1] : -1; }
#define MakeResourceOrdinalW(ord)  (WCHAR [2]){ [0] = (WCHAR)-1, [1] = (ord) }

#endif /* _ADDIN_H */
