#ifndef _INTRIN_H
#define _INTRIN_H

/* intrin.h - private header for compiler intrinsic functions */

#if __POCC__ >= 500
#pragma once
#endif

#include <xmmintr.h>

/* type definitions */
#ifndef _SIZE_T_DEFINED
#define _SIZE_T_DEFINED
#if __POCC_TARGET__ == 3
typedef unsigned long long size_t;
#else
typedef unsigned int size_t;
#endif
#endif /* _SIZE_T_DEFINED */

#ifndef _WCHAR_T_DEFINED
#define _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#endif /* _WCHAR_T_DEFINED */

/* data declarations */
#ifdef _WINNT_H
extern IMAGE_DOS_HEADER __ImageBase;
#endif /* _WINNT_H */

/* constants for __prefetch() */
#define _MM_HINT_T0   1
#define _MM_HINT_T1   2
#define _MM_HINT_T2   3
#define _MM_HINT_NTA  0

#undef _INTRIN_ALL_
#undef _INTRIN_X86_
#undef _INTRIN_X86_AMD64_
#undef _INTRIN_AMD64_
#undef _INTRIN_ARM_
#undef _INTRIN_NIL_
#undef _INTRIN_DEF_

#define _INTRIN_NIL_(N)  /* not so much, no */
#define _INTRIN_DEF_(N)  N;

#if __POCC_TARGET__ == 1
#define _INTRIN_ALL_        _INTRIN_DEF_
#define _INTRIN_X86_        _INTRIN_DEF_
#define _INTRIN_X86_AMD64_  _INTRIN_DEF_
#define _INTRIN_AMD64_      _INTRIN_NIL_
#define _INTRIN_ARM_        _INTRIN_NIL_
#elif __POCC_TARGET__ == 2
#define _INTRIN_ALL_        _INTRIN_DEF_
#define _INTRIN_X86_        _INTRIN_NIL_
#define _INTRIN_X86_AMD64_  _INTRIN_NIL_
#define _INTRIN_AMD64_      _INTRIN_NIL_
#define _INTRIN_ARM_        _INTRIN_DEF_
#elif __POCC_TARGET__ == 3
#define _INTRIN_ALL_        _INTRIN_DEF_
#define _INTRIN_X86_        _INTRIN_NIL_
#define _INTRIN_X86_AMD64_  _INTRIN_DEF_
#define _INTRIN_AMD64_      _INTRIN_DEF_
#define _INTRIN_ARM_        _INTRIN_NIL_
#else
#define _INTRIN_ALL_        _INTRIN_NIL_
#define _INTRIN_X86_        _INTRIN_NIL_
#define _INTRIN_X86_AMD64_  _INTRIN_NIL_
#define _INTRIN_AMD64_      _INTRIN_NIL_
#define _INTRIN_ARM_        _INTRIN_NIL_
#endif /* __POCC_TARGET__ */

/* declarations */
_INTRIN_X86_AMD64_(size_t __cdecl strlen(const char *))
_INTRIN_X86_(char * __cdecl strcpy(char * restrict, const char * restrict))
_INTRIN_X86_(int __cdecl strcmp(const char *, const char *))
_INTRIN_X86_AMD64_(void * __cdecl memcpy(void * restrict, const void * restrict, size_t))
_INTRIN_X86_(void * __cdecl memset(void *, int, size_t))
_INTRIN_X86_AMD64_(size_t __cdecl wcslen(const wchar_t *))
_INTRIN_X86_(wchar_t * __cdecl wcscpy(wchar_t * restrict, const wchar_t * restrict))

/* more secure C functions (TR24731-1) */
#if __STDC_WANT_LIB_EXT1__ && !defined(_WINCE)
_INTRIN_AMD64_(size_t __cdecl strnlen_s(const char *, size_t))
_INTRIN_AMD64_(size_t __cdecl wcsnlen_s(const wchar_t *, size_t))
#endif /* __STDC_WANT_LIB_EXT1__ */

_INTRIN_ALL_(int __cdecl abs(int))
_INTRIN_ALL_(long __cdecl labs(long))
_INTRIN_AMD64_(long long __cdecl llabs(long long))
_INTRIN_X86_AMD64_(double __cdecl fabs(double))
_INTRIN_X86_AMD64_(float __cdecl fabsf(float))
_INTRIN_X86_AMD64_(long double __cdecl fabsl(long double))
_INTRIN_X86_AMD64_(double __cdecl sqrt(double))
_INTRIN_X86_AMD64_(float __cdecl sqrtf(float))
_INTRIN_X86_AMD64_(long double __cdecl sqrtl(long double))
_INTRIN_X86_(double __cdecl exp(double))
_INTRIN_X86_(float __cdecl expf(float))
_INTRIN_X86_(long double __cdecl expl(long double))
_INTRIN_X86_(double __cdecl log(double))
_INTRIN_X86_(float __cdecl logf(float))
_INTRIN_X86_(long double __cdecl logl(long double))
_INTRIN_X86_(double __cdecl log10(double))
_INTRIN_X86_(float __cdecl log10f(float))
_INTRIN_X86_(long double __cdecl log10l(long double))
_INTRIN_X86_(double __cdecl log2(double))
_INTRIN_X86_(float __cdecl log2f(float))
_INTRIN_X86_(long double __cdecl log2l(long double))
_INTRIN_X86_(double __cdecl sin(double))
_INTRIN_X86_(float __cdecl sinf(float))
_INTRIN_X86_(long double __cdecl sinl(long double))
_INTRIN_X86_(double __cdecl cos(double))
_INTRIN_X86_(float __cdecl cosf(float))
_INTRIN_X86_(long double __cdecl cosl(long double))
_INTRIN_X86_(double __cdecl tan(double))
_INTRIN_X86_(float __cdecl tanf(float))
_INTRIN_X86_(long double __cdecl tanl(long double))
_INTRIN_X86_(double __cdecl atan(double))
_INTRIN_X86_(float __cdecl atanf(float))
_INTRIN_X86_(long double __cdecl atanl(long double))
_INTRIN_X86_(long __cdecl lrint(double))
_INTRIN_X86_(long __cdecl lrintf(float))
_INTRIN_X86_(long __cdecl lrintl(long double))
_INTRIN_X86_(long long __cdecl llrint(double))
_INTRIN_X86_(long long __cdecl llrintf(float))
_INTRIN_X86_(long long __cdecl llrintl(long double))

_INTRIN_X86_AMD64_(double __cdecl creal(double _Complex))
_INTRIN_X86_AMD64_(float __cdecl crealf(float _Complex))
_INTRIN_X86_AMD64_(long double __cdecl creall(long double _Complex))
_INTRIN_X86_AMD64_(double __cdecl cimag(double _Complex))
_INTRIN_X86_AMD64_(float __cdecl cimagf(float _Complex))
_INTRIN_X86_AMD64_(long double __cdecl cimagl(long double _Complex))
_INTRIN_X86_AMD64_(double _Complex __cdecl conj(double _Complex))
_INTRIN_X86_AMD64_(float _Complex __cdecl conjf(float _Complex))
_INTRIN_X86_AMD64_(long double _Complex __cdecl conjl(long double _Complex))

_INTRIN_AMD64_(unsigned long long __cdecl _llrotl(unsigned long long, int))
_INTRIN_AMD64_(unsigned long long __cdecl _llrotr(unsigned long long, int))
_INTRIN_ALL_(unsigned long __cdecl _lrotl(unsigned long, int))
_INTRIN_ALL_(unsigned long __cdecl _lrotr(unsigned long, int))
_INTRIN_ALL_(unsigned int __cdecl _rotl(unsigned int, int))
_INTRIN_ALL_(unsigned int __cdecl _rotr(unsigned int, int))
_INTRIN_X86_AMD64_(int __cdecl _ffs(int))

_INTRIN_ALL_(void __cdecl __debugbreak(void))
_INTRIN_X86_AMD64_(void * __cdecl _ReturnAddress(void))
_INTRIN_X86_AMD64_(void * __cdecl _AddressOfReturnAddress(void))

_INTRIN_X86_(volatile unsigned int __cdecl __getflags(void))
_INTRIN_ARM_(volatile unsigned int __cdecl __getflags(void))
_INTRIN_AMD64_(volatile unsigned long long __cdecl __getflags(void))

_INTRIN_ALL_(unsigned int __cdecl _bswap(unsigned int))
_INTRIN_AMD64_(unsigned long long __cdecl _bswap64(unsigned long long))
_INTRIN_X86_AMD64_(void __cdecl _cpuid(int *, int))
_INTRIN_X86_AMD64_(int __cdecl _inp(unsigned short))
_INTRIN_X86_AMD64_(unsigned short __cdecl _inpw(unsigned short))
_INTRIN_X86_AMD64_(unsigned long __cdecl _inpd(unsigned short))
_INTRIN_X86_AMD64_(int __cdecl _outp(unsigned short, int))
_INTRIN_X86_AMD64_(unsigned short __cdecl _outpw(unsigned short, unsigned short))
_INTRIN_X86_AMD64_(unsigned long __cdecl _outpd(unsigned short, unsigned long))
_INTRIN_X86_AMD64_(unsigned long long __cdecl _rdtsc(void))
#ifdef _MSC_EXTENSIONS
_INTRIN_X86_AMD64_(unsigned long long __cdecl __rdtsc(void))
#endif /* _MSC_EXTENSIONS */
_INTRIN_X86_AMD64_(void __cdecl _enable(void))
_INTRIN_X86_AMD64_(void __cdecl _disable(void))

_INTRIN_ALL_(void * __cdecl _alloca(size_t))

_INTRIN_X86_AMD64_(unsigned char __cdecl _bittest(long const *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _bittestandcomplement(long *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _bittestandset(long *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _bittestandreset(long *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _interlockedbittestandset(long *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _interlockedbittestandreset(long *, long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _BitScanForward(unsigned long *, unsigned long))
_INTRIN_X86_AMD64_(unsigned char __cdecl _BitScanReverse(unsigned long *, unsigned long))
_INTRIN_X86_AMD64_(long __cdecl _InterlockedExchange(long volatile *, long))
_INTRIN_X86_AMD64_(long __cdecl _InterlockedExchangeAdd(long volatile *, long))
_INTRIN_X86_AMD64_(void * __cdecl _InterlockedExchangePointer(void * volatile *, void *))
_INTRIN_X86_AMD64_(long __cdecl _InterlockedIncrement(long volatile *))
_INTRIN_X86_AMD64_(long __cdecl _InterlockedDecrement(long volatile *))
_INTRIN_AMD64_(unsigned char __cdecl _bittest64(long long const *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _bittestandcomplement64(long long *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _bittestandset64(long long *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _bittestandreset64(long long *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _interlockedbittestandset64(long long *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _interlockedbittestandreset64(long long *, long long))
_INTRIN_AMD64_(unsigned char __cdecl _BitScanForward64(unsigned long *, unsigned long long))
_INTRIN_AMD64_(unsigned char __cdecl _BitScanReverse64(unsigned long *, unsigned long long))
_INTRIN_AMD64_(long long __cdecl _InterlockedExchange64(long long volatile *, long long))
_INTRIN_AMD64_(long long __cdecl _InterlockedExchangeAdd64(long long volatile *, long long))
_INTRIN_AMD64_(long long __cdecl _InterlockedIncrement64(long long volatile *))
_INTRIN_AMD64_(long long __cdecl _InterlockedDecrement64(long long volatile *))

_INTRIN_X86_(unsigned char __cdecl __readfsbyte(unsigned long))
_INTRIN_X86_(unsigned short __cdecl __readfsword(unsigned long))
_INTRIN_X86_(unsigned long __cdecl __readfsdword(unsigned long))
_INTRIN_X86_(void __cdecl __writefsbyte(unsigned long, unsigned char))
_INTRIN_X86_(void __cdecl __writefsword(unsigned long, unsigned short))
_INTRIN_X86_(void __cdecl __writefsdword(unsigned long, unsigned long))

_INTRIN_AMD64_(unsigned char __cdecl __readgsbyte(unsigned long))
_INTRIN_AMD64_(unsigned short __cdecl __readgsword(unsigned long))
_INTRIN_AMD64_(unsigned long __cdecl __readgsdword(unsigned long))
_INTRIN_AMD64_(unsigned long long __cdecl __readgsqword(unsigned long))
_INTRIN_AMD64_(void __cdecl __writegsbyte(unsigned long, unsigned char))
_INTRIN_AMD64_(void __cdecl __writegsword(unsigned long, unsigned short))
_INTRIN_AMD64_(void __cdecl __writegsdword(unsigned long, unsigned long))
_INTRIN_AMD64_(void __cdecl __writegsqword(unsigned long, unsigned long long))

/* (re)move later */
_INTRIN_X86_AMD64_(void __cdecl _mm_prefetch(char *, int))
_INTRIN_X86_AMD64_(void __cdecl _mm_clflush(const void *))
_INTRIN_X86_AMD64_(void __cdecl _mm_lfence(void))
_INTRIN_X86_AMD64_(void __cdecl _mm_sfence(void))
_INTRIN_X86_AMD64_(void __cdecl _mm_mfence(void))
_INTRIN_X86_AMD64_(void __cdecl _mm_pause(void))

_INTRIN_ARM_(void __cdecl _PreLoad(unsigned long *))

#endif /* _INTRIN_H */
