#ifndef _STDIO_H
#define _STDIO_H

/* stdio.h - C99 standard header */

#if __POCC__ >= 500
#pragma once
#endif

#ifndef _CRTIMP
#ifdef _DLL
#define _CRTIMP  __declspec(dllimport)
#else
#define _CRTIMP
#endif
#endif /* _CRTIMP */

/* macros */
#ifndef NULL
#define NULL  ((void *)0)
#endif

#define EOF  (-1)

#define FILENAME_MAX  260

#ifndef _WINCE

#define _IOFBF  0
#define _IOLBF  1
#define _IONBF  2

#define BUFSIZ  512

#define FOPEN_MAX  256

#define L_tmpnam  16
#define TMP_MAX  32

#endif /* _WINCE */

#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2

#ifndef _WINCE
#define stdin   (&__stdin)
#define stdout  (&__stdout)
#define stderr  (&__stderr)
#else /* _WINCE */
#define stdin  _getstdfilex(0)
#define stdout  _getstdfilex(1)
#define stderr  _getstdfilex(2)
#endif /* _WINCE */

/* type definitions */
#ifndef _MBSTATE_T_DEFINED
#define _MBSTATE_T_DEFINED
typedef struct mbstate_t {
    unsigned long wchar;
    unsigned short rsrv, state;
} mbstate_t;
#endif /* _MBSTATE_T_DEFINED */

#ifndef _SIZE_T_DEFINED
#define _SIZE_T_DEFINED
#if __POCC_TARGET__ == 3
typedef unsigned long long size_t;
#else
typedef unsigned int size_t;
#endif
#endif /* _SIZE_T_DEFINED */

#ifndef _WCHAR_T_DEFINED
#define _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#endif /* _WCHAR_T_DEFINED */

#ifndef _VA_LIST_DEFINED
#define _VA_LIST_DEFINED
typedef char *va_list;
#endif /* _VA_LIST_DEFINED */

#ifndef _OFF_T_DEFINED
#define _OFF_T_DEFINED
#if __POCC_TARGET__ == 3
typedef long long _off_t;
#else
typedef long _off_t;
#endif /* __POCC_TARGET */
#ifdef __POCC__OLDNAMES
#if __POCC_TARGET__ == 3
typedef long long off_t;
#else
typedef long off_t;
#endif /* __POCC_TARGET */
#endif /* __POCC__OLDNAMES */
#endif /* _OFF_T_DEFINED */

#ifndef _WINCE
typedef struct FILE {
#if __POCC__ >= 500
    unsigned int mode;
#else
    unsigned short mode;
    unsigned short pad;
#endif /* __POCC__ */
    int fh;
    unsigned char *buf, *bufend, *ptr;
    unsigned char *getend, *putend, *backptr;
    wchar_t *wbackptr, wbackbuf[2];
    unsigned char *getback, *wgetend, *wputend;
    mbstate_t wstate;
    char *tmpnam;
    unsigned char backbuf[8], cbuf;
#if __POCC__ >= 500
    int locknum;
#endif /* __POCC__ >= 500 */
} FILE;

typedef struct fpos_t {
#if __POCC__ >= 500
    long long off;
#else /* __POCC__ < 500 */
    long off;
#endif /* __POCC__ < 500 */
    mbstate_t wstate;
} fpos_t;
#else /* _WINCE */
typedef void FILE;
typedef long fpos_t;
#endif /* _WINCE */

/* operations on files */
int __cdecl remove(const char *);  /* WINCE: in crtce.lib */
int __cdecl rename(const char *, const char *);  /* WINCE: in crtce.lib */
#ifndef _WINCE
FILE * __cdecl tmpfile(void);
char * __cdecl tmpnam(char *);
wchar_t * __cdecl _wtmpnam(wchar_t *);
#endif /* _WINCE */

/* file access functions */
int __cdecl fclose(FILE *);
int __cdecl fflush(FILE *);
FILE * __cdecl fopen(const char * restrict, const char * restrict);
int __cdecl setvbuf(FILE * restrict, char * restrict, int, size_t);
#ifndef _WINCE
FILE * __cdecl freopen(const char * restrict, const char * restrict, FILE * restrict);
void __cdecl setbuf(FILE * restrict, char * restrict);
#endif /* _WINCE */

/* formatted input/output functions */
int __cdecl fprintf(FILE * restrict, const char * restrict, ...);
int __cdecl fscanf(FILE * restrict, const char * restrict, ...);
int __cdecl sprintf(char * restrict, const char * restrict, ...);
int __cdecl sscanf(const char * restrict, const char * restrict, ...);
int __cdecl vfprintf(FILE * restrict, const char * restrict, va_list);
int __cdecl vsprintf(char * restrict, const char * restrict, va_list);
int __cdecl vprintf(const char * restrict, va_list);
int __cdecl printf(const char * restrict, ...);
int __cdecl scanf(const char * restrict, ...);
#ifndef _WINCE
int __cdecl vfscanf(FILE * restrict, const char * restrict, va_list);
int __cdecl vsscanf(const char * restrict, const char * restrict, va_list);
int __cdecl vscanf(const char * restrict, va_list);
int __cdecl snprintf(char * restrict, size_t, const char * restrict, ...);
int __cdecl vsnprintf(char * restrict, size_t, const char * restrict, va_list);
#else /* _WINCE */
#define snprintf  _snprintf
#define vsnprintf  _vsnprintf
#endif /* _WINCE */

/* character input/output functions */
int __cdecl fgetc(FILE *);
char * __cdecl fgets(char * restrict, int, FILE * restrict);
int __cdecl fputc(int, FILE *);
int __cdecl fputs(const char * restrict, FILE * restrict);
int __cdecl getchar(void);
char * __cdecl gets(char *);
int __cdecl putchar(int);
int __cdecl puts(const char *);
int __cdecl ungetc(int, FILE *);
#ifndef _WINCE
int __cdecl getc(FILE *);
int __cdecl putc(int, FILE *);
#else /* _WINCE */
#define getc(str)  fgetc(str)
#define putc(c,str)  fputc(c,str)
#endif /* _WINCE */

/* direct input/output functions */
size_t __cdecl fread(void * restrict, size_t, size_t, FILE * restrict);
size_t __cdecl fwrite(const void * restrict, size_t, size_t, FILE * restrict);

/* file positioning functions */
int __cdecl fgetpos(FILE * restrict, fpos_t * restrict);
int __cdecl fseek(FILE *, long, int);
int __cdecl fsetpos(FILE *, const fpos_t *);
long __cdecl ftell(FILE *);
void __cdecl rewind(FILE *);  /* WINCE: in crtce.lib */

/* error-handling functions */
void __cdecl clearerr(FILE *);
int __cdecl feof(FILE *);
int __cdecl ferror(FILE *);
#ifndef _WINCE
void __cdecl perror(const char *);
void __cdecl _wperror(const wchar_t *);
#endif /* _WINCE */

/* private extensions to standard C */
int __cdecl _fileno(FILE *);
int __cdecl _fcloseall(void);
FILE * __cdecl _wfopen(const wchar_t * restrict, const wchar_t * restrict);
FILE * __cdecl _wfreopen(const wchar_t * restrict, const wchar_t * restrict, FILE * restrict);
#ifndef _WINCE
FILE * __cdecl _fdopen(int, const char *);
FILE * __cdecl _wfdopen(int, const wchar_t *);
FILE * __cdecl _popen(const char *, const char *);  /* 06-03-13 */
int __cdecl _pclose(FILE *);  /* 06-03-13 */
#else /* _WINCE */
FILE * __cdecl _getstdfilex(int);
#endif /* _WINCE */
int __cdecl _snprintf(char * restrict, size_t, const char * restrict, ...);
int __cdecl _vsnprintf(char * restrict, size_t, const char * restrict, va_list);
int __cdecl _wremove(const wchar_t *);  /* WINCE: in crtce.lib */
int __cdecl _wrename(const wchar_t *, const wchar_t *);  /* WINCE: in crtce.lib */
#ifndef _WINCE
int __cdecl _fseeko(FILE *, _off_t, int);
_off_t __cdecl _ftello(FILE *);
#endif /* _WINCE */

#ifndef _WINCE

/* data declarations */
extern _CRTIMP FILE __stdin, __stdout, __stderr;
#if !defined(_DLL)
extern FILE *__filetab[FOPEN_MAX];
#endif

/* macro overrides */
#if !defined(__MT__) && !defined(_DLL)
#define getc(str)     ((str)->ptr < (str)->getend ? *(str)->ptr++ : (fgetc)(str))
#define getchar()     (__filetab[0]->ptr < __filetab[0]->getend ? *__filetab[0]->ptr++ : (fgetc)(__filetab[0]))
#define putc(c,str)   ((str)->ptr < (str)->putend ? (*(str)->ptr++ = c) : (fputc)(c, str))
#define putchar(c)    (__filetab[1]->ptr < __filetab[1]->putend ? (*__filetab[1]->ptr++ = c) : (fputc)(c, __filetab[1]))
#define _fileno(str)  ((str)->fh)
#endif

#endif /* _WINCE */

/* compatibility names */
#ifdef __POCC__OLDNAMES
int __cdecl fcloseall(void);
FILE * __cdecl fdopen(int, const char *);
int __cdecl fileno(FILE *);
FILE * __cdecl popen(const char *, const char *);
int __cdecl pclose(FILE *);
#ifndef _WINCE
int __cdecl fseeko(FILE *, off_t, int);
off_t __cdecl ftello(FILE *);
#endif /* _WINCE */
#endif /* __POCC__OLDNAMES */

/* more secure C functions (TR24731-1) */
#if __STDC_WANT_LIB_EXT1__ && !defined(_WINCE)

#define L_tmpnam_s  L_tmpnam
#define TMP_MAX_S  TMP_MAX

#ifndef _ERRNO_T_DEFINED
#define _ERRNO_T_DEFINED
typedef int errno_t;
#endif /* _ERRNO_T_DEFINED */

#ifndef _RSIZE_T_DEFINED
#define _RSIZE_T_DEFINED
#if __POCC_TARGET__ == 3
typedef unsigned long long rsize_t;
#else
typedef unsigned int rsize_t;
#endif
#endif /* _RSIZE_T_DEFINED */

/* operations on files */
errno_t __cdecl tmpfile_s(FILE * restrict * restrict);
errno_t __cdecl tmpnam_s(char *, rsize_t);

/* file access functions */
errno_t __cdecl fopen_s(FILE * restrict * restrict, const char * restrict, const char * restrict);
errno_t __cdecl freopen_s(FILE * restrict * restrict, const char * restrict, const char * restrict, FILE * restrict);

/* formatted input/output functions */
int __cdecl fprintf_s(FILE * restrict, const char * restrict, ...);
int __cdecl fscanf_s(FILE * restrict, const char * restrict, ...);
int __cdecl printf_s(const char * restrict, ...);
int __cdecl scanf_s(const char * restrict, ...);
int __cdecl snprintf_s(char * restrict, rsize_t, const char * restrict, ...);
int __cdecl sprintf_s(char * restrict, rsize_t, const char * restrict, ...);
int __cdecl sscanf_s(const char * restrict, const char * restrict, ...);
int __cdecl vfprintf_s(FILE * restrict, const char * restrict, va_list);
int __cdecl vfscanf_s(FILE * restrict, const char * restrict, va_list);
int __cdecl vprintf_s(const char * restrict, va_list);
int __cdecl vscanf_s(const char * restrict, va_list);
int __cdecl vsnprintf_s(char * restrict, rsize_t, const char * restrict, va_list);
int __cdecl vsprintf_s(char * restrict, rsize_t, const char * restrict, va_list);
int __cdecl vsscanf_s(const char * restrict, const char * restrict, va_list);

/* character input/output functions */
char * __cdecl gets_s(char *, rsize_t);

#endif /* __STDC_WANT_LIB_EXT1__ */

#endif /* _STDIO_H */
