#ifndef _WCTYPE_H
#define _WCTYPE_H

/* wctype.h - C99 standard header */

#if __POCC__ >= 500
#pragma once
#endif

/* macros */
#define WEOF  ((wint_t)(-1))

#ifndef _WINCE

/* type definitions */
#ifndef _WINT_T_DEFINED
#define _WINT_T_DEFINED
typedef unsigned short wint_t;
#endif /* _WINT_T_DEFINED */

typedef unsigned int wctrans_t;
typedef unsigned int wctype_t;

/* declarations */
int __cdecl iswalnum(wint_t);
int __cdecl iswalpha(wint_t);
int __cdecl iswcntrl(wint_t);
int __cdecl iswdigit(wint_t);
int __cdecl iswgraph(wint_t);
int __cdecl iswlower(wint_t);
int __cdecl iswprint(wint_t);
int __cdecl iswpunct(wint_t);
int __cdecl iswspace(wint_t);
int __cdecl iswblank(wint_t);
int __cdecl iswupper(wint_t);
int __cdecl iswxdigit(wint_t);
int __cdecl _iswascii(wint_t);
int __cdecl iswctype(wint_t, wctype_t);
wint_t __cdecl towlower(wint_t);
wint_t __cdecl towupper(wint_t);
wint_t __cdecl towctrans(wint_t, wctrans_t);
wctrans_t __cdecl wctrans(const char *);
wctype_t __cdecl wctype(const char *);

/* internal stuff */
int __cdecl __iswctype(wint_t, wctype_t);
wint_t __cdecl __towctrans(wint_t, wctrans_t);

/* macro overrides */
#define iswalnum(wc)  __iswctype(wc,1)
#define iswalpha(wc)  __iswctype(wc,2)
#define iswcntrl(wc)  __iswctype(wc,3)
#define iswdigit(wc)  __iswctype(wc,4)
#define iswgraph(wc)  __iswctype(wc,5)
#define iswlower(wc)  __iswctype(wc,6)
#define iswprint(wc)  __iswctype(wc,7)
#define iswpunct(wc)  __iswctype(wc,8)
#define iswspace(wc)  __iswctype(wc,9)
#define iswblank(wc)  __iswctype(wc,12)
#define iswupper(wc)  __iswctype(wc,10)
#define iswxdigit(wc)  __iswctype(wc,11)
#define _iswascii(wc)  __iswctype(wc,13)
#define iswctype(wc,desc)  __iswctype(wc,desc)
#define towlower(wc)  __towctrans(wc,1)
#define towupper(wc)  __towctrans(wc,2)
#define towctrans(wc,desc)  __towctrans(wc,desc)

#ifdef __POCC__OLDNAMES
int __cdecl iswascii(wint_t);
#define iswascii(c)  __iswctype(wc,13)
#endif /* __POCC__OLDNAMES */

#else /* _WINCE */

/* type definitions */
#ifndef _WINT_T_DEFINED
#define _WINT_T_DEFINED
typedef unsigned short wint_t;
#endif /* _WINT_T_DEFINED */

#ifndef _WCTYPE_T_DEFINED
#define _WCTYPE_T_DEFINED
typedef unsigned short wctype_t;
#endif /* _WCTYPE_T_DEFINED */

/* classification bit masks */
#define _WCE_DIGIT  0x01    /* decimal digit */
#define _WCE_HEX    0x02    /* hexadecimal digit */
#define _WCE_LOWER  0x04    /* lower case letter */
#define _WCE_UPPER  0x08    /* upper case letter */
#define _WCE_CNTRL  0x10    /* control character */
#define _WCE_WHITE  0x20    /* white space character */
#define _WCE_SPACE  0x40    /* space */
#define _WCE_PUNCT  0x80    /* punctuation character */
#define _WCE_BLANK  0x100   /* blank (tab) */
#define _WCE_ASCII  0x200   /* ascii */
#define _WCE_ALPHA  (0x0100|_WCE_UPPER|_WCE_LOWER)  /* alphabetic character */

/* declarations */
int iswctype(wint_t, wctype_t);
wint_t __cdecl towlower(wint_t);
wint_t __cdecl towupper(wint_t);

/* macro overrides */
#define iswalnum(c)  (iswctype((c), _WCE_ALPHA|_WCE_DIGIT))
#define iswalpha(c)  (iswctype((c), _WCE_ALPHA))
#define iswcntrl(c)  (iswctype((c), _WCE_CNTRL))
#define iswdigit(c)  (iswctype((c), _WCE_DIGIT))
#define iswgraph(c)  (iswctype((c), _WCE_PUNCT|_WCE_ALPHA|_WCE_DIGIT))
#define iswlower(c)  (iswctype((c), _WCE_LOWER))
#define iswprint(c)  (iswctype((c), _WCE_BLANK|_WCE_PUNCT|_WCE_ALPHA|_WCE_DIGIT))
#define iswpunct(c)  (iswctype((c), _WCE_PUNCT))
#define iswspace(c)  (iswctype((c), _WCE_SPACE))
#define iswblank(c)  ((c) == L'\t' || (c) == L' ')
#define iswupper(c)  (iswctype((c), _WCE_UPPER))
#define iswxdigit(c)  (iswctype((c), _WCE_HEX))
#define _iswascii(c)  ((unsigned)(c) < 0x80)
/* towctrans() */

#ifdef __POCC__OLDNAMES
#define iswascii  _iswascii
#endif /* __POCC__OLDNAMES */

#endif /* _WINCE */

#endif /* _WCTYPE_H */
