#include <stdio.h>
#include <string.h>
#include <gc.h>
#include "bitstring.h"
// Bits per Word
#define BPW (8*sizeof(unsigned long))
// Number of words needed for a given bit count
#define nwords(len) ((((len) + BPW-1)&(~(BPW-1)))/BPW)

int bytesizeFromBitLen(int bitlen)
{
	int bytesiz = (7+bitlen) >> 3;
	bytesiz = (3+bytesiz) & ~3;
	return bytesiz;
}

Bitstring *new_bitstring(size_t bitlen)
{
	Bitstring *set;
	int bytesiz;

	set = 	GC_malloc(sizeof(Bitstring));
	if (set == NULL)
		return NULL;
	set->count = bitlen;
	bytesiz = bytesizeFromBitLen(bitlen);
	set->Data = GC_malloc(bytesiz);
	if (set->Data == NULL	) {
		set = NULL;
	}
	else {
		memset(set->Data,0,bytesiz);
	}
	return set;
}

Bitstring *set_size_bitstring(Bitstring *b,size_t bitlen)
{
	int bytesize,bs;
	size_t oldsize;
	char *oldData;
	if (b == NULL)
		return NULL;
	bytesize = bytesizeFromBitLen(bitlen);
	oldsize = bytesizeFromBitLen(b->count);
	bs = bytesizeFromBitLen(b->count);
	oldData = b->Data;
	b->Data = GC_malloc(bytesize);
	if (b->Data == NULL) {
		b->Data = oldData;
		return NULL;
	}
	memset(b->Data,0,bytesize);
	bs = (bs < oldsize) ? bs : oldsize;
	memcpy(b->Data,oldData,bs);
	b->count = bitlen;
	return b;
}


void delete_bitstring(Bitstring *b)
{
	b->Data = NULL;
	b->count = 0;
}

Bitstring *copy_bitstring(Bitstring *b)
{
	Bitstring *result;

	if (b == NULL)
		return NULL;
	result = GC_malloc(sizeof(Bitstring));
	memcpy(result,b,sizeof(Bitstring));
	result->Data = GC_malloc((8+b->count) & 7);
	memcpy(result->Data,b->Data,(8+b->count)&7);
	return result;
}

size_t get_size_bitstring(Bitstring *b)
{
	return (b == NULL) ? 0 : b->count;
}


bool __declspec(naked) set_element_bitstring(Bitstring *bs,size_t position,bool b)
{
}
bool operator []=(Bitstring *bs,size_t position,bool b)
{
	if (bs == NULL)
		return 0;
	if (position >= bs->count)
		return b;
	if (b) {
		bs->Data[position >> 3] |= 1 << (position&7);
	}
	else
		bs->Data[position >> 3] &= ~(1 << (position&7));
	return b;
}

bool __declspec(naked) get_element_bitstring(Bitstring *bs,size_t position)
{
}
bool operator [](Bitstring *bs,size_t position)
{
	if (position >= bs->count)
		return 0;
	return ((bs->Data[position >> 3] >> (position&7))&1);
}

Bitstring * __declspec(naked) shiftright_bitstring(Bitstring *bs,int shift)
{
}
Bitstring * operator>>(Bitstring *bs,int shift)
{
	unsigned int left = 0,tmp,tmp1,len,bytesize;
	unsigned char *pdata;

	len = bs->count;
	pdata = bs->Data;
	bytesize = (7+len)>>3;
	if (shift > len) {
		memset(bs->Data,0,bytesize);
		return bs;
	}
	if (shift >= 8) {
		left = shift >> 3;
		memmove(pdata,pdata+left,bytesize-left);
		shift &= 7;
		memset(pdata+bytesize-left,0,left);
	}
	if (shift) {
		while (len >= 8) {
			tmp = *pdata;
				tmp >>= shift;
			tmp1 = pdata[1];
			tmp1 <<= (8 - shift);
			tmp |= tmp1;
			*pdata = tmp;
			pdata++;
			len -= 8;
		}
		if (len) {
			tmp = *pdata;
			tmp >>= shift;
			*pdata = tmp;
		}
	}
	return bs;
}

Bitstring * __declspec(naked) shiftleft_bitstring(Bitstring *bs,int shift)
{
}
Bitstring * operator<<(Bitstring *bs,int shift)
{
	unsigned int left = 0,tmp,tmp1,len,bytesize;
	unsigned char *pdata;

	len = bs->count;
	pdata = bs->Data;
	bytesize = (7+len)>>3;
	if (shift > len) {
		memset(bs->Data,0,bytesize);
		return bs;
	}
	if (shift >= 8) {
		left = shift >> 3;
		memmove(pdata+left,pdata,bytesize-left);
		shift &= 7;
		memset(pdata,0,left);
	}
	if (shift) {
		while (len >= 8) {
			tmp = *pdata;
				tmp <<= shift;
			tmp1 = pdata[1];
			tmp1 >>= (8 - shift);
			tmp |= tmp1;
			*pdata = tmp;
			pdata++;
			len -= 8;
		}
		if (len) {
			tmp = *pdata;
			tmp <<= shift;
			*pdata = tmp;
		}
	}
	return bs;
}

Bitstring *get_subrange_bitstring(Bitstring *bs,size_t start,size_t end)
{
	size_t t,len;
	Bitstring *result;
	int startbyte,endbyte,shiftamount;

	if (bs == NULL)
		return NULL;
	if (end >= bs->count)
		end = bs->count;
	if (start > end) {
		t = start;
		start = end;
		end = t;
	}
	len  = end-start;
	result = new_bitstring(len);
	startbyte = start >> 3;
	endbyte = (7+end) >> 3;
	memcpy(result->Data,bs->Data+startbyte,endbyte-startbyte);
	shiftamount = start&0x7;
	if (shiftamount)
		shiftright_bitstring(result,shiftamount);
	return result;
}

int __declspec(naked) less_equal_bitstring(Bitstring bsl,Bitstring bsr)
{
}
int operator <=(Bitstring bsl,Bitstring bsr)
{
	unsigned int *pdataL, *pdataR;
	int i,j,k,l;

	if ((bsl.count == 0 && bsr.count != 0) ||
		(bsl.count == bsr.count && bsr.count == 0))
		return 1;
	if (bsr.count == 0 && bsl.count != 0)
		return 0;
	pdataL = (unsigned int *)bsl.Data;
	pdataR = (unsigned int *)bsr.Data;
	i = bytesizeFromBitLen(bsl.count) >> 3;
	j = bytesizeFromBitLen(bsr.count) >> 3;
	l = k = (i < j) ? i : j;
	while (--k >= 0) {
		if (pdataL[k] &  ~pdataR[k])
			return 0;
	}
	if (i == j)
		return 1;
	// The rest must be zero
	k = (i < j) ? j : i;
	pdataL = (i < j) ? pdataR : pdataL;
	while (l < k) {
		if (pdataL[l])
			return 0;
		l++;
	}
	return 1;
}

int __declspec(naked) equal_bitstring(Bitstring bsl,Bitstring bsr)
{
}
int operator ==(Bitstring bsl,Bitstring bsr)
{
	unsigned int *pdataL,*pdataR;
	int top,i;

	if (bsl.count != bsr.count)
		return 0;
	pdataL = (unsigned int *)bsl.Data;
	pdataR = (unsigned int *)bsr.Data;
	top = bytesizeFromBitLen(bsl.count) >> 3;
	i = 0;
	while (i < top) {
		if (pdataL[i] != pdataR[i])
			return 0;
	}
	return 1;
}

Bitstring * operator | (Bitstring *bsl,Bitstring *bsr)
{
	size_t len,i,resultlen;
	Bitstring *result,*src;

	len = bsl->count < bsr->count ? bsl->count : bsr->count;
	resultlen = bsl->count < bsr->count ? bsr->count : bsl->count;
	result = new_bitstring(resultlen);
	for (i=0; i<len;i++) {
		result->Data[i] = bsl->Data[i] | bsr->Data[i];
	}
	if (bsl->count < bsr->count)
		src = bsr;
	else
		src = bsl;
	while (i < resultlen) {
		result->Data[i] = src->Data[i];
		i++;
	}
	return result;
}

Bitstring * operator & (Bitstring *bsl,Bitstring *bsr)
{
	size_t len,i,resultlen;
	Bitstring *result,*src;

	len = bsl->count < bsr->count ? bsl->count : bsr->count;
	resultlen = bsl->count < bsr->count ? bsr->count : bsl->count;
	result = new_bitstring(resultlen);
	for (i=0; i<len;i++) {
		result->Data[i] = bsl->Data[i] & bsr->Data[i];
	}
	if (bsl->count < bsr->count)
		src = bsr;
	else
		src = bsl;
	while (i < resultlen) {
		result->Data[i] = src->Data[i];
		i++;
	}
	return result;
}

#define makeToBitstring(T) Bitstring  * overloaded ToBitstring(T a)\
{\
    Bitstring *result = new_bitstring(8*sizeof(T));\
	memcpy(result->Data,&a,sizeof(T));\
	return result;\
}

makeToBitstring(int)
makeToBitstring(double)
makeToBitstring(long double)
makeToBitstring(long long)

#undef makeToBitstring

Bitstring * overloaded ToBitstring(char * str)
{
	int i=0;
	Bitstring *result;
	char *save = str;
	if (str == NULL)
		return NULL;
	while (*str) {
		while (*str == ' ' || *str == '\t')
			str++;
		if (*str == 0)
			break;
		if (*str == '1' || *str == '0') {
			i++;
			str++;
		}
		else break;
	}
	if (i == 0)
		return NULL;
	result = new_bitstring(i);
	str = save;
	i = 0;
	while (*str) {
		while (*str == ' ' || *str == '\t')
			str++;
		if (*str == 0)
			break;
		if (*str == '1' || *str == '0') {
			if (*str == '1')
				result->Data[i >> 3] |= 1 << i&7;
			str++;
			i++;
		}
		else break;
	}
	return result;
}

int sprint_bitstring(Bitstring *b,size_t bufsiz,char *out)
{
	int i,result=0;
	char *top = out+bufsiz-1;

	for (i=b->count-1; i >= 0; i--) {
		if (out >= top)
			break;
		if ((b->Data[i >> 3] >> (i&7))&1)
			*out = '1';
		else
			*out = '0';
		out++;
		if (out >= top)
			break;
		*out++ = ' ';
		result += 2;
	}
	*out = 0;
	return result;
}

int fprint_bitstring(Bitstring *b,FILE *out)
{
	int i,result=0;

	for (i=b->count-1; i >= 0; i--) {
		if ((b->Data[i >> 3] >> (i&7))&1)
			fputc('1',out);
		else
			fputc('0',out);
		fputc(' ',out);
		result += 2;
	}
	return result;
}

Bitstring *concat_bitstring(Bitstring *b1,Bitstring *b2)
{
	Bitstring *result;
	int bytesize,i,idx;

	if (b1 == NULL)
		return copy_bitstring(b2);
	if (b2 == NULL)
		return copy_bitstring(b1);
	result = new_bitstring(b1->count+b2->count);
	if (result == NULL)
		return NULL;
	bytesize = bytesizeFromBitLen(b1->count);
	memcpy(result->Data,b1->Data,bytesize);
	bytesize = bytesizeFromBitLen(result->count);
	for (i=b1->count;i<bytesize;i++) {
		idx = i-b1->count;
		if ((b2->Data[idx >>3] >>(idx&7))&1) {
			result->Data[i >>3] |= 1 << (i&7);
		}
	}
	return result;
}

Bitstring *reverse_bitstring(Bitstring *b)
{
	Bitstring *result;
	int i,j;

	if (b == NULL)
		return NULL;
	result = new_bitstring(b->count);
	i = 0;
	j = b->count - 1;
	while (i < result->count) {
		if ((b->Data[j >> 3] >> (j&7))&1) {
			result->Data[i >> 3] |= 1 << (i&7);
		}
		i++;
		j--;
	}
	return result;
}

int Sizeof_bitstring(Bitstring *b)
{
	int result;
	if (b == NULL)
		return 0;
	result = sizeof(Bitstring);
	result += bytesizeFromBitLen(b->count);
	return result;
}

